@@ -165,7 +165,7 @@
 ** *** empty log message ***
 **
 ** Revision 1.34  2009/11/27 15:07:25  gerlof
-** Give up root-priviliges at a earlier stage.
+** Give up root-privileges at a earlier stage.
 **
 ** Revision 1.33  2009/11/27 14:01:01  gerlof
 ** Introduce system-wide configuration file /etc/atoprc
@@ -271,8 +271,6 @@
 **
 */
 
-static const char rcsid[] = "$Id: atop.c,v 1.49 2010/10/23 14:01:00 gerlof Exp $";
-
 #include <sys/types.h>
 #include <sys/param.h>
 #include <sys/mman.h>
@@ -316,12 +314,13 @@ int		linelen  = 80;
 char		acctreason;	/* accounting not active (return val) 	*/
 char		rawname[RAWNAMESZ];
 char		rawreadflag;
-unsigned int	begintime, endtime;
+time_t		begintime, endtime, cursortime;	// epoch or time in day
 char		flaglist[MAXFL];
 char		deviatonly = 1;
 char      	usecolors  = 1;  /* boolean: colors for high occupation  */
 char		threadview = 0;	 /* boolean: show individual threads     */
 char      	calcpss    = 0;  /* boolean: read/calculate process PSS  */
+char      	getwchan   = 0;  /* boolean: obtain wchan string         */
 
 unsigned short	hertz;
 unsigned int	pagesize;
@@ -457,8 +456,8 @@ main(int argc, char *argv[])
 	struct rlimit	rlim;
 
 	/*
-	** since priviliged actions will be done later on, at this stage
-	** the root-priviliges are dropped by switching effective user-id
+	** since privileged actions will be done later on, at this stage
+	** the root-privileges are dropped by switching effective user-id
 	** to real user-id (security reasons)
 	*/
         if (! droprootprivs() )
@@ -533,9 +532,24 @@ main(int argc, char *argv[])
 				break;
 
 			   case 'r':		/* reading of raw data ?      */
-				if (optind < argc && *(argv[optind]) != '-')
-					strncpy(rawname, argv[optind++],
-							RAWNAMESZ-1);
+				if (optind < argc)
+				{
+					if (*(argv[optind]) == '-')
+					{
+						if (strlen(argv[optind]) == 1)
+						{
+							strcpy(rawname,
+								"/dev/stdin");
+							optind++;
+						}
+					}
+					else
+					{
+						strncpy(rawname, argv[optind],
+								RAWNAMESZ-1);
+						optind++;
+					}
+				}
 
 				rawreadflag++;
 				break;
@@ -544,21 +558,13 @@ main(int argc, char *argv[])
 				midnightflag++;
 				break;
 
-                           case 'a':		/* all processes per sample ? */
-				deviatonly = 0;
-				break;
-
-                           case 'R':		/* all processes per sample ? */
-				calcpss = 1;
-				break;
-
                            case 'b':		/* begin time ?               */
-				if ( !hhmm2secs(optarg, &begintime) )
+				if ( !getbranchtime(optarg, &begintime) )
 					prusage(argv[0]);
 				break;
 
                            case 'e':		/* end   time ?               */
-				if ( !hhmm2secs(optarg, &endtime) )
+				if ( !getbranchtime(optarg, &endtime) )
 					prusage(argv[0]);
 				break;
 
@@ -576,6 +582,18 @@ main(int argc, char *argv[])
 				linelen = atoi(optarg);
 				break;
 
+                           case MALLPROC:	/* all processes per sample ? */
+				deviatonly = 0;
+				break;
+
+                           case MCALCPSS:	/* calculate PSS per sample ? */
+				calcpss = 1;
+				break;
+
+                           case MGETWCHAN:	/* obtain wchan string?       */
+				getwchan = 1;
+				break;
+
 			   default:		/* gather other flags */
 				flaglist[i++] = c;
 			}
@@ -652,15 +670,15 @@ main(int argc, char *argv[])
 	signal(SIGTERM, cleanstop);
 
 	/*
-	** regain the root-priviliges that we dropped at the beginning
-	** to do some priviliged work
+	** regain the root-privileges that we dropped at the beginning
+	** to do some privileged work
 	*/
 	regainrootprivs();
 
 	/*
 	** lock ATOP in memory to get reliable samples (also when
 	** memory is low and swapping is going on);
-	** ignored if not running under superuser priviliges!
+	** ignored if not running under superuser privileges!
 	*/
 	rlim.rlim_cur	= RLIM_INFINITY;
 	rlim.rlim_max	= RLIM_INFINITY;
@@ -671,11 +689,13 @@ main(int argc, char *argv[])
 	/*
 	** increment CPU scheduling-priority to get reliable samples (also
 	** during heavy CPU load);
-	** ignored if not running under superuser priviliges!
+	** ignored if not running under superuser privileges!
 	*/
 	if ( nice(-20) == -1)
 		;
 
+	set_oom_score_adj();
+
 	/*
 	** switch-on the process-accounting mechanism to register the
 	** (remaining) resource-usage by processes which have finished
@@ -694,7 +714,7 @@ main(int argc, char *argv[])
 
 	/*
 	** since privileged activities are finished now, there is no
-	** need to keep running under root-priviliges, so switch
+	** need to keep running under root-privileges, so switch
 	** effective user-id to real user-id
 	*/
         if (! droprootprivs() )
@@ -1040,7 +1060,7 @@ prusage(char *myname)
 	printf("\t\tor\n");
 	printf("Usage: %s -w  file  [-S] [-%c] [interval [samples]]\n",
 					myname, MALLPROC);
-	printf("       %s -r [file] [-b hh:mm] [-e hh:mm] [-flags]\n",
+	printf("       %s -r [file] [-b [YYYYMMDD]hhmm] [-e [YYYYMMDD]hhmm] [-flags]\n",
 					myname);
 	printf("\n");
 	printf("\tgeneric flags:\n");
@@ -1049,21 +1069,24 @@ prusage(char *myname)
 	                "only)\n", MALLPROC);
 	printf("\t  -%c  calculate proportional set size (PSS) per process\n", 
 	                MCALCPSS);
+	printf("\t  -%c  determine WCHAN (string) per thread\n", MGETWCHAN);
 	printf("\t  -P  generate parseable output for specified label(s)\n");
 	printf("\t  -L  alternate line length (default 80) in case of "
 			"non-screen output\n");
 
-	(*vis.show_usage)();
+	if (vis.show_usage)
+		(*vis.show_usage)();
 
 	printf("\n");
 	printf("\tspecific flags for raw logfiles:\n");
 	printf("\t  -w  write raw data to   file (compressed)\n");
 	printf("\t  -r  read  raw data from file (compressed)\n");
-	printf("\t      special file: y[y...] for yesterday (repeated)\n");
+	printf("\t      symbolic file: y[y...] for yesterday (repeated)\n");
+	printf("\t      file name '-': read raw data from stdin\n");
 	printf("\t  -S  finish atop automatically before midnight "
 	                "(i.s.o. #samples)\n");
-	printf("\t  -b  begin showing data from specified time\n");
-	printf("\t  -e  finish showing data after specified time\n");
+	printf("\t  -b  begin showing data from specified date/time\n");
+	printf("\t  -e  finish showing data after specified date/time\n");
 	printf("\n");
 	printf("\tinterval: number of seconds   (minimum 0)\n");
 	printf("\tsamples:  number of intervals (minimum 1)\n");
@@ -1151,7 +1174,10 @@ readrc(char *path, int syslevel)
 
 			i = strlen(linebuf);
 
-			if (i > 0 && linebuf[i-1] == '\n')
+			if (i <= 1)	// empty line?
+				continue;
+
+			if (linebuf[i-1] == '\n')
 				linebuf[i-1] = 0;
 
 			nr = sscanf(linebuf, "%19s %255[^#]",
