@@ -22,7 +22,7 @@
 ** See the GNU General Public License for more details.
 */
 #define	EQ		0
-#define SECSDAY		86400
+#define SECONDSINDAY	86400
 #define RAWNAMESZ	256
 
 /*
@@ -80,9 +80,10 @@ extern char		deviatonly;
 extern char		usecolors;
 extern char		threadview;
 extern char		calcpss;
+extern char		getwchan;
 extern char		rawname[];
 extern char		rawreadflag;
-extern unsigned int	begintime, endtime;
+extern time_t		begintime, endtime, cursortime;	// epoch or time in day
 extern char		flaglist[];
 extern struct visualize vis;
 
@@ -137,8 +138,9 @@ void		generic_usage(void);
 int		atopsar(int, char *[]);
 char   		*convtime(time_t, char *);
 char   		*convdate(time_t, char *);
-int   		hhmm2secs(char *, unsigned int *);
-int		daysecs(time_t);
+int   		getbranchtime(char *, time_t *);
+time_t		normalize_epoch(time_t, long);
+
 
 char   		*val2valstr(count_t, char *, int, int, int);
 char   		*val2memstr(count_t, char *, int, int, int);
@@ -191,3 +193,4 @@ unsigned int	netatop_exitstore(void);
 void		netatop_exiterase(void);
 void		netatop_exithash(char);
 void		netatop_exitfind(unsigned long, struct tstat *, struct tstat *);
+void		set_oom_score_adj(void);
