@@ -74,22 +74,33 @@
 /*
 ** Semaphore-handling
 **
-** Two semaphore groups are created with one semaphore each.
+** Two semaphore groups are created:
 **
 ** The private semaphore (group) specifies the number of atopacctd processes
 ** running (to be sure that only one daemon is active at the time).
 **
-** The public semaphore (group) reflects the number of processes using
-** the process accounting shadow files, i.e. the number of clients. This
-** semaphore starts at a high value and should be decremented whenever a
-** client starts using the shadow files and incremented again whenever that
-** client terminates.
+** The public semaphore group contains two semaphores:
+**
+**   0: the number of processes using the process accounting shadow files,
+**	i.e. the number of (atop) clients. This semaphore starts at a high
+**	value and should be decremented by every (atop) client that starts
+**	using the shadow files and incremented again whenever that (atop)
+**	client terminates.
+**
+**   1:	binary semphore that has to be claimed before using/modifying
+**	semaphore 0 in this group.
 */
 static int		semprv;
+
+
 static int		sempub;
 #define SEMTOTAL        100
 #define	NUMCLIENTS	(SEMTOTAL - semctl(sempub, 0, GETVAL, 0))
 
+struct sembuf   semlocknowait = {1, -1, SEM_UNDO|IPC_NOWAIT},
+                semunlock     = {1, +1, SEM_UNDO};
+
+
 static char		atopacctdversion[] = ATOPVERS;
 static char		atopacctddate[]    = ATOPDATE;
 
@@ -134,6 +145,7 @@ main(int argc, char *argv[])
 	time_t			gclast = time(0);
 
 	struct sigaction	sigcleanup;
+	int			liResult;
 
 	/*
 	** argument passed?
@@ -247,18 +259,20 @@ main(int argc, char *argv[])
 		}
 	}
 
-	if ( (sempub = semget(PACCTPUBKEY, 0, 0)) == -1)	// not existing?
+	// create new semaphore group
+	//
+	if ( (sempub = semget(PACCTPUBKEY, 0, 0)) != -1)	// existing?
+		(void) semctl(sempub, 0, IPC_RMID, 0);
+
+	if ( (sempub = semget(PACCTPUBKEY, 2, 0666|IPC_CREAT|IPC_EXCL)) >= 0)
 	{
-		if ( (sempub = semget(PACCTPUBKEY, 1,
-						0666|IPC_CREAT|IPC_EXCL)) >= 0)
-		{
-			(void) semctl(sempub, 0, SETVAL, SEMTOTAL);
-		}
-		else
-		{
-			perror("cannot create public semaphore");
-			exit(3);
-		}
+		(void) semctl(sempub, 0, SETVAL, SEMTOTAL);
+		(void) semctl(sempub, 1, SETVAL, 1);
+	}
+	else
+	{
+		perror("cannot create public semaphore");
+		exit(3);
 	}
 
 	/*
@@ -304,7 +318,17 @@ main(int argc, char *argv[])
 
 	umask(022);
 
-	(void) chdir("/tmp");			// go to a safe place
+	liResult = chdir("/tmp");			// go to a safe place
+
+	if(liResult != 0)
+	{
+		char lcMessage[64];
+
+		snprintf(lcMessage, sizeof(lcMessage) - 1,
+		          "%s:%d - Error %d changing to tmp dir\n", 
+		          __FILE__, __LINE__, errno );
+		fprintf(stderr, "%s", lcMessage);
+	}
 
 	/*
 	** increase semaphore to define that atopacctd is running
@@ -322,7 +346,7 @@ main(int argc, char *argv[])
 	*/
 	snprintf(accountpath, sizeof accountpath, "%s/%s", pacctdir, PACCTORIG);
 
-	(void) unlink(accountpath);
+	(void) unlink(accountpath);	// in case atopacctd previously killed
 
 	if ( (afd = creat(accountpath, 0600)) == -1)
        	{
@@ -345,9 +369,29 @@ main(int argc, char *argv[])
 
 	/*
  	** create directory to store the shadow files
+	** when atopacctd was previously killed, rename the
+	** old directory to a unique name
 	*/
 	snprintf(shadowdir, sizeof shadowdir, "%s/%s", pacctdir, PACCTSHADOWD);
 
+	if ( stat(shadowdir, &dirstat) == 0 )	// already exists?
+	{
+		if (S_ISDIR(dirstat.st_mode))	// and is directory?
+		{
+			// define new name to save directory
+			char newshadow[256];
+
+			snprintf(newshadow, sizeof newshadow, "%s-old-%d",
+							shadowdir, getpid());
+
+			if ( rename(shadowdir, newshadow) == -1)
+			{
+				perror("could not rename old shadow directory");
+				exit(5);
+			}
+		}
+	}
+
 	if ( mkdir(shadowdir, 0755) == -1)
        	{
 		perror(shadowdir);
@@ -367,8 +411,9 @@ main(int argc, char *argv[])
 	/*
 	** raise priority (be sure the nice value becomes -20,
 	** independent of the current nice value)
+	** this may fail without notice for non-privileged processes
 	*/
-	(void) nice(-39);
+	liResult = nice(-39);
 
 	/*
  	** connect to NETLINK socket of kernel to be triggered
@@ -422,6 +467,7 @@ main(int argc, char *argv[])
 	while (! cleanup_and_go)
 	{
 		int	state;
+		time_t	curtime;
 
 		/*
 		** await termination of (at least) one process and
@@ -434,10 +480,18 @@ main(int argc, char *argv[])
 			break;
 
 		/*
- 		** verify if garbage collection is needed
-		** i.e. removal of shadow files that are not in use any more
+ 		** garbage collection (i.e. removal of shadow files that
+		** are not in use any more) is needed in case:
+		**
+		** - shadow files are currently maintained because
+		**   at least one atop is running, and
+		** - shadow files have not been removed for GCINTERVAL
+		**   seconds, or
+		** - the system clock has been modified (lowered)
 		*/
-		if ( shadowbusy && time(0) > gclast + GCINTERVAL )
+		if ( shadowbusy &&
+			(time(&curtime) > gclast + GCINTERVAL ||
+		               curtime  < gclast                ) )
 		{
 			gcshadows(&oldshadow, curshadow);
 			gclast = time(0);
@@ -664,33 +718,39 @@ awaitprocterm(int nfd, int afd, int sfd, char *accountpath,
 	** have been using the shadow files till now and
 	** cleanup has to be performed
 	*/
-	if (NUMCLIENTS == 0)
+	if (semop(sempub, &semlocknowait, 1) == 0) 	// lock succeeded?
 	{
-		/*
-		** did last client just disappeared?
-		*/
-		if (*shadowbusyp)
+		if (NUMCLIENTS == 0)
 		{
 			/*
-			** remove all shadow files
+			** did last client just disappear?
 			*/
-			gcshadows(oldshadowp, (*curshadowp)+1);
-			*oldshadowp = 0;
-			*curshadowp = 0;
-			stotsize    = 0;
-
-			/*
- 			** create new file with sequence 0
-			*/
-			(void) close(sfd);
-
-			sfd = createshadow(*curshadowp);
-			setcurrent(*curshadowp);
-
-			*shadowbusyp = 0;
+			if (*shadowbusyp)
+			{
+				/*
+				** remove all shadow files
+				*/
+				gcshadows(oldshadowp, (*curshadowp)+1);
+				*oldshadowp = 0;
+				*curshadowp = 0;
+				stotsize    = 0;
+	
+				/*
+ 				** create new file with sequence 0
+				*/
+				(void) close(sfd);
+	
+				sfd = createshadow(*curshadowp);
+				setcurrent(*curshadowp);
+	
+				*shadowbusyp = 0;
+			}
+	
+			(void) semop(sempub, &semunlock, 1);
+			return 1;
 		}
 
-		return 1;
+		(void) semop(sempub, &semunlock, 1);
 	}
 
 	*shadowbusyp = 1;
@@ -966,6 +1026,7 @@ setcurrent(long curshadow)
 	static int	cfd = -1;
 	char		currentpath[128], currentdata[128];
 	int		len;
+	int		liResult;
 
 	/*
 	** assemble file name of currency file and open (only once)
@@ -992,9 +1053,31 @@ setcurrent(long curshadow)
 	/*
 	** wipe currency file and write new assembled string
 	*/
-	(void) ftruncate(cfd, 0);
+	liResult = ftruncate(cfd, 0);
+
+	if(liResult != 0)
+	{
+		char lcMessage[64];
+
+		snprintf(lcMessage, sizeof(lcMessage) - 1,
+		          "%s:%d - Error %d ftruncate\n\n", __FILE__, __LINE__,
+		          errno );
+		fprintf(stderr, "%s", lcMessage);
+	}
+
 	(void) lseek(cfd, 0, SEEK_SET);
-	(void) write(cfd, currentdata, len);
+
+	liResult = write(cfd, currentdata, len);
+
+	if(liResult == -1)
+	{
+		char lcMessage[64];
+
+		snprintf(lcMessage, sizeof(lcMessage) - 1,
+		          "%s:%d - Error %d writing\n\n", __FILE__, __LINE__,
+		          errno );
+		fprintf(stderr, "%s", lcMessage);
+	}
 }
 
 /*
