@@ -11,7 +11,7 @@
 ** E-mail:      gerlof.langeveld@atoptool.nl
 ** Initial:     July/August 2018
 ** --------------------------------------------------------------------------
-** Copyright (C) 2018-2019 Gerlof Langeveld
+** Copyright (C) 2018-2020 Gerlof Langeveld
 **
 ** This program is free software; you can redistribute it and/or modify it
 ** under the terms of the GNU General Public License as published by the
@@ -51,6 +51,7 @@
 #include "atop.h"
 #include "photosyst.h"
 #include "photoproc.h"
+#include "rawlog.h"
 
 #include "prev/photosyst_20.h"
 #include "prev/photoproc_20.h"
@@ -70,6 +71,9 @@
 #include "prev/photosyst_25.h"
 #include "prev/photoproc_25.h"
 
+#include "prev/photosyst_26.h"
+#include "prev/photoproc_26.h"
+
 
 ///////////////////////////////////////////////////////////////
 // Conversion functions
@@ -214,6 +218,33 @@ tgen_to_22(void *old, void *new, count_t oldsize, count_t newsize)
 	g22->vpid = 0;
 }
 
+void
+tcpu_to_26(void *old, void *new, count_t oldsize, count_t newsize)
+{
+	// unused values appear not to be zeroed in version 2.5
+	//
+	struct cpu_25	*c25 = old;
+	struct cpu_26	*c26 = new;
+
+	memcpy(c26, c25, sizeof *c26);		// copy entire struct
+
+        memset(&(c26->wchan), 0, sizeof c26->wchan);
+	c26->rundelay = 0;
+}
+
+void
+tmem_to_26(void *old, void *new, count_t oldsize, count_t newsize)
+{
+	// unused values appear not to be zeroed in version 2.5
+	//
+	struct mem_25	*m25 = old;
+	struct mem_26	*m26 = new;
+
+	memcpy(m26, m25, sizeof *m26);		// copy entire struct
+
+	m26->vlock = 0;
+}
+
 ///////////////////////////////////////////////////////////////
 // conversion definition for various structs in sstat and tstat
 //
@@ -238,6 +269,7 @@ struct sstat_22		sstat_22;
 struct sstat_23		sstat_23;
 struct sstat_24		sstat_24;
 struct sstat_25		sstat_25;
+struct sstat_26		sstat_26;
 struct sstat		sstat;
 
 struct tstat_20		tstat_20;
@@ -246,6 +278,7 @@ struct tstat_22		tstat_22;
 struct tstat_23		tstat_23;
 struct tstat_24		tstat_24;
 struct tstat_25		tstat_25;
+struct tstat_26		tstat_26;
 struct tstat		tstat;
 
 struct convertall {
@@ -454,6 +487,36 @@ struct convertall {
 		{sizeof(struct gpu_25),
 			STROFFSET(&tstat_25.gpu, &tstat_25),	justcopy},
 	},
+
+	{SETVERSION(2,6), // 2.5 --> 2.6
+		 sizeof(struct sstat_26),	&sstat_26,
+		 sizeof(struct tstat_26), 	NULL,
+
+		{sizeof(struct cpustat_26),  	&sstat_26.cpu,	justcopy},
+		{sizeof(struct memstat_26),  	&sstat_26.mem,	justcopy},
+		{sizeof(struct netstat_26),  	&sstat_26.net,	justcopy},
+		{sizeof(struct intfstat_26), 	&sstat_26.intf,	justcopy},
+		{sizeof(struct dskstat_26),  	&sstat_26.dsk,	justcopy},
+		{sizeof(struct nfsstat_26),  	&sstat_26.nfs,	justcopy},
+		{sizeof(struct contstat_26), 	&sstat_26.cfs,	justcopy},
+		{sizeof(struct wwwstat_26),  	&sstat_26.www,	justcopy},
+		{sizeof(struct pressure_26),  	&sstat_26.psi,	justcopy},
+		{sizeof(struct gpustat_26),  	&sstat_26.gpu,	justcopy},
+		{sizeof(struct ifbstat_26),  	&sstat_26.ifb,	justcopy},
+
+		{sizeof(struct gen_26),
+			STROFFSET(&tstat_26.gen, &tstat_26),	justcopy},
+		{sizeof(struct cpu_26),
+			STROFFSET(&tstat_26.cpu, &tstat_26),	tcpu_to_26},
+		{sizeof(struct dsk_26),
+			STROFFSET(&tstat_26.dsk, &tstat_26),	justcopy},
+		{sizeof(struct mem_26),
+			STROFFSET(&tstat_26.mem, &tstat_26),	tmem_to_26},
+		{sizeof(struct net_26),
+			STROFFSET(&tstat_26.net, &tstat_26),	justcopy},
+		{sizeof(struct gpu_26),
+			STROFFSET(&tstat_26.gpu, &tstat_26),	justcopy},
+	},
 };
 
 int	numconvs = sizeof convs / sizeof(struct convertall);
@@ -462,69 +525,6 @@ int	numconvs = sizeof convs / sizeof(struct convertall);
 // End of conversion functions
 ///////////////////////////////////////////////////////////////
 
-
-/*
-** structure which describes the raw file contents
-**
-** layout raw file:    rawheader
-**
-**                     rawrecord                           \
-**                     compressed system-level statistics   | sample 1
-**                     compressed process-level statistics /
-**
-**                     rawrecord                           \
-**                     compressed system-level statistics   | sample 2
-**                     compressed process-level statistics /
-**
-** etcetera .....
-*/
-#define	MYMAGIC		(unsigned int) 0xfeedbeef
-
-struct rawheader {
-	unsigned int	magic;
-
-	unsigned short	aversion;	/* creator atop version with MSB */
-	unsigned short	future1;	/* can be reused 		 */
-	unsigned short	future2;	/* can be reused 		 */
-	unsigned short	rawheadlen;	/* length of struct rawheader    */
-	unsigned short	rawreclen;	/* length of struct rawrecord    */
-	unsigned short	hertz;		/* clock interrupts per second   */
-	unsigned short	sfuture[6];	/* future use                    */
-	unsigned int	sstatlen;	/* length of struct sstat        */
-	unsigned int	tstatlen;	/* length of struct tstat        */
-	struct utsname	utsname;	/* info about this system        */
-	char		cfuture[8];	/* future use                    */
-
-	unsigned int	pagesize;	/* size of memory page (bytes)   */
-	int		supportflags;  	/* used features                 */
-	int		osrel;		/* OS release number             */
-	int		osvers;		/* OS version number             */
-	int		ossub;		/* OS version subnumber          */
-	int		ifuture[6];	/* future use                    */
-};
-
-struct rawrecord {
-	time_t		curtime;	/* current time (epoch)         */
-
-	unsigned short	flags;		/* various flags                */
-	unsigned short	sfuture[3];	/* future use                   */
-
-	unsigned int	scomplen;	/* length of compressed sstat   */
-	unsigned int	pcomplen;	/* length of compressed tstat's */
-	unsigned int	interval;	/* interval (number of seconds) */
-	unsigned int	ndeviat;	/* number of tasks in list      */
-	unsigned int	nactproc;	/* number of processes in list  */
-	unsigned int	ntask;		/* total number of tasks        */
-	unsigned int    totproc;	/* total number of processes	*/
-	unsigned int    totrun;		/* number of running  threads	*/
-	unsigned int    totslpi;	/* number of sleeping threads(S)*/
-	unsigned int    totslpu;	/* number of sleeping threads(D)*/
-	unsigned int	totzomb;	/* number of zombie processes   */
-	unsigned int	nexit;		/* number of exited processes   */
-	unsigned int	noverflow;	/* number of overflow processes */
-	unsigned int	ifuture[6];	/* future use                   */
-};
-
 // function prototypes
 //
 static int	openin(char *);
