@@ -29,8 +29,6 @@
 ** --------------------------------------------------------------------------
 */
 
-static const char rcsid[] = "$Id: atopsar.c,v 1.28 2010/11/26 06:19:43 gerlof Exp $";
-
 #include <sys/types.h>
 #include <sys/param.h>
 #include <sys/mman.h>
@@ -156,19 +154,23 @@ atopsar(int argc, char *argv[])
 				break;
 
                            case 'b':		/* begin time ?          */
-				if ( !hhmm2secs(optarg, &begintime) )
+				if ( !getbranchtime(optarg, &begintime) )
 					pratopsaruse(argv[0]);
 
 				saved_begintime = begintime;
 				break;
 
                            case 'e':		/* end   time ?          */
-				if ( !hhmm2secs(optarg, &endtime) )
+				if ( !getbranchtime(optarg, &endtime) )
 					pratopsaruse(argv[0]);
 				break;
 
 			   case 'r':		/* reading of file data ? */
 				strncpy(rawname, optarg, RAWNAMESZ-1);
+
+				if (strcmp(rawname, "-") == 0)
+					strcpy(rawname, "/dev/stdin");
+
 				rawreadflag++;
 				break;
 
@@ -354,8 +356,8 @@ atopsar(int argc, char *argv[])
 	curtime = time(0);
 
         /*
-	** regain the root-priviliges that we dropped at the beginning
-	** to do some priviliged work now
+	** regain the root-privileges that we dropped at the beginning
+	** to do some privileged work now
 	*/
         regainrootprivs();
 
@@ -1004,7 +1006,7 @@ pratopsaruse(char *myname)
 	int	i;
 
 	fprintf(stderr,
-		"Usage: %s [-flags] [-r file|date|y...] [-R cnt] [-b hh:mm] [-e hh:mm]\n",
+		"Usage: %s [-flags] [-r file|-|date|y...] [-R cnt] [-b time] [-e time]\n",
 								myname);
 	fprintf(stderr, "\t\tor\n");
 	fprintf(stderr,
@@ -1018,13 +1020,13 @@ pratopsaruse(char *myname)
 	fprintf(stderr,
 		"\t  -r  read statistical data from specific atop logfile\n");
 	fprintf(stderr,
-		"\t      (pathname, or date in format YYYYMMDD, or y[y..])\n");
+		"\t      (pathname, - for stdin, date in format YYYYMMDD, or y[y..])\n");
 	fprintf(stderr,
 		"\t  -R  summarize <cnt> samples into one sample\n");
 	fprintf(stderr,
-		"\t  -b  begin  showing data from  specified time\n");
+		"\t  -b  begin  showing data from  specified time as [YYYYMMDD]hhmm\n");
 	fprintf(stderr,
-		"\t  -e  finish showing data after specified time\n");
+		"\t  -e  finish showing data after specified time as [YYYYMMDD]hhmm\n");
 	fprintf(stderr,
 		"\t  -S  print timestamp on every line in case of more "
 		"resources\n");
@@ -1132,11 +1134,11 @@ postprint(unsigned int badness)
 }
 
 /*
-** function the handle the default flags for atopsar as
-** read from the file ~/.atoprc
+** function to handle the default flags for atopsar as
+** read from the files ~/.atoprc and /etc/atoprc
 */
 void
-do_atopsarflags(char *val)
+do_atopsarflags(char *name, char *val)
 {
 	int     i, j;
 
@@ -1563,13 +1565,12 @@ swapline(struct sstat *ss, struct tstat *ts, struct tstat **ps, int nactproc,
 }
 
 /*
-** swapping statistics
+** PSI statistics
 */
 static void
 psihead(int osvers, int osrel, int ossub)
 {
-	printf("cs_10_60_300   ms_10_60_300 mf_10_60_300   "
-	       "is_10_60_300 if_10_60_300");
+	printf("cpusome    memsome  memfull    iosome  iofull");
 }
 
 static int
@@ -1578,25 +1579,52 @@ psiline(struct sstat *ss, struct tstat *ts, struct tstat **ps, int nactproc,
         int osvers, int osrel, int ossub, char *tstamp,
         int ppres,  int ntrun, int ntslpi, int ntslpu, int pexit, int pzombie)
 {
+	// calculate pressure percentages for entire interval
+	unsigned int	csperc  = ss->psi.cpusome.total/(deltatic*10000/hz);
+	unsigned int	msperc  = ss->psi.memsome.total/(deltatic*10000/hz);
+	unsigned int	mfperc  = ss->psi.memfull.total/(deltatic*10000/hz);
+	unsigned int	isperc  = ss->psi.iosome.total /(deltatic*10000/hz);
+	unsigned int	ifperc  = ss->psi.iofull.total /(deltatic*10000/hz);
+	unsigned int	badness = 0;
+
 	if (!ss->psi.present)
 	{
-		printf("no PSI stats available for this system .....\n");
-		return 0;
+		printf("no PSI stats available for this interval...\n");
+		return 1;
 	}
 
-	printf("%3.0f%%%3.0f%%%3.0f%%   %3.0f%%%3.0f%%%3.0f%% "
-	       "%3.0f%%%3.0f%%%3.0f%%   %3.0f%%%3.0f%%%3.0f%% "
-	       "%3.0f%%%3.0f%%%3.0f%%\n",
-		ss->psi.cpusome.avg10, ss->psi.cpusome.avg60,
-		ss->psi.cpusome.avg300,
-		ss->psi.memsome.avg10, ss->psi.memsome.avg60,
-		ss->psi.memsome.avg300,
-		ss->psi.memfull.avg10, ss->psi.memfull.avg60,
-		ss->psi.memfull.avg300,
-		ss->psi.iosome.avg10, ss->psi.iosome.avg60,
-		ss->psi.iosome.avg300,
-		ss->psi.iofull.avg10, ss->psi.iofull.avg60,
-		ss->psi.iofull.avg300);
+	// correct percentages if needed
+	if (csperc > 100)
+		csperc = 100;
+
+	if (msperc > 100)
+		msperc = 100;
+
+	if (mfperc > 100)
+		mfperc = 100;
+
+	if (isperc > 100)
+		isperc = 100;
+
+	if (ifperc > 100)
+		ifperc = 100;
+
+	// consider a 'some' percentage > 0 as almost critical
+	// (I/O full tends to increase rapidly as well)
+	if (csperc || msperc || isperc || ifperc)
+		badness = 80;
+
+	// consider a memory 'full' percentage > 0 as critical
+	if (mfperc)
+		badness = 100;
+
+	// show results
+	preprint(badness);
+
+	printf("   %3u%%       %3u%%     %3u%%      %3u%%    %3u%%",
+		csperc, msperc, mfperc, isperc, ifperc);
+
+	postprint(badness);
 
 	return 1;
 }
@@ -1690,7 +1718,7 @@ gendskline(struct sstat *ss, char *tstamp, char selector)
 			pn = dp->name;
 
 		printf("%-14s %3.0lf%% %6.1lf %7.1lf %7.1lf %7.1lf "
-		       "%5.1lf %6.2lf ms",
+		       "%5.1lf %9.5lf ms",
 		    	pn,
 			mstot ? (double)dp->io_ms  *  100.0 / mstot   : 0.0,
 			mstot ? (double)dp->nread  * 1000.0 / mstot   : 0.0,
