@@ -168,8 +168,6 @@
 **
 */
 
-static const char rcsid[] = "$Id: deviate.c,v 1.45 2010/10/23 14:02:03 gerlof Exp $";
-
 #include <sys/types.h>
 #include <sys/param.h>
 #include <sys/stat.h>
@@ -584,6 +582,11 @@ calcdiff(struct tstat *devstat, struct tstat *curstat, struct tstat *prestat,
 	devstat->cpu.curcpu   = curstat->cpu.curcpu;
 	devstat->cpu.sleepavg = curstat->cpu.sleepavg;
 
+	if (curstat->cpu.wchan[0])
+		strcpy(devstat->cpu.wchan, curstat->cpu.wchan);
+	else
+		devstat->cpu.wchan[0] = 0;
+
 	devstat->mem.vexec    = curstat->mem.vexec;
 	devstat->mem.vmem     = curstat->mem.vmem;
 	devstat->mem.rmem     = curstat->mem.rmem;
@@ -592,6 +595,7 @@ calcdiff(struct tstat *devstat, struct tstat *curstat, struct tstat *prestat,
 	devstat->mem.vstack   = curstat->mem.vstack;
 	devstat->mem.vlibs    = curstat->mem.vlibs;
 	devstat->mem.vswap    = curstat->mem.vswap;
+	devstat->mem.vlock    = curstat->mem.vlock;
 
 	if (curstat->gpu.state || prestat->gpu.state) // GPU use?
 	{
@@ -636,6 +640,8 @@ calcdiff(struct tstat *devstat, struct tstat *curstat, struct tstat *prestat,
 	if (devstat->cpu.utime > totusedcpu)
 		devstat->cpu.utime = 1;
 
+	devstat->cpu.rundelay  =
+		subcount(curstat->cpu.rundelay, prestat->cpu.rundelay);
 	/*
 	** do further calculations
 	*/
@@ -808,6 +814,7 @@ deviatsyst(struct sstat *cur, struct sstat *pre, struct sstat *dev,
 	dev->mem.cachedrt	= cur->mem.cachedrt;
 	dev->mem.totswap	= cur->mem.totswap;
 	dev->mem.freeswap	= cur->mem.freeswap;
+	dev->mem.swapcached	= cur->mem.swapcached;
 
 	dev->mem.shmem		= cur->mem.shmem;
 	dev->mem.shmrss		= cur->mem.shmrss;
@@ -818,6 +825,7 @@ deviatsyst(struct sstat *cur, struct sstat *pre, struct sstat *dev,
 	dev->mem.hugepagesz	= cur->mem.hugepagesz;
 
 	dev->mem.vmwballoon	= cur->mem.vmwballoon;
+	dev->mem.zfsarcsize	= cur->mem.zfsarcsize;
 
 	dev->mem.swouts		= subcount(cur->mem.swouts,  pre->mem.swouts);
 	dev->mem.swins		= subcount(cur->mem.swins,   pre->mem.swins);
@@ -1528,6 +1536,7 @@ totalsyst(char category, struct sstat *new, struct sstat *tot)
 		tot->mem.cachedrt	 = new->mem.cachedrt;
 		tot->mem.totswap	 = new->mem.totswap;
 		tot->mem.freeswap	 = new->mem.freeswap;
+		tot->mem.swapcached	 = new->mem.swapcached;
 
 		tot->mem.shmem		 = new->mem.shmem;
 		tot->mem.shmrss		 = new->mem.shmrss;
