@@ -75,6 +75,7 @@
 #include <unistd.h>
 #include <stdlib.h>
 #include <string.h>
+#include <limits.h>
 #include <sys/utsname.h>
 
 #include "atop.h"
@@ -421,7 +422,7 @@ print_GPU(char *hp, struct sstat *ss, struct tstat *ps, int nact)
 void
 print_MEM(char *hp, struct sstat *ss, struct tstat *ps, int nact)
 {
-	printf(	"%s %u %lld %lld %lld %lld %lld %lld %lld %lld %lld %lld %lld %lld %lld %lld\n",
+	printf(	"%s %u %lld %lld %lld %lld %lld %lld %lld %lld %lld %lld %lld %lld %lld %lld %lld\n",
 			hp,
 			pagesize,
 			ss->mem.physmem,
@@ -437,7 +438,8 @@ print_MEM(char *hp, struct sstat *ss, struct tstat *ps, int nact)
         		ss->mem.shmswp,
         		ss->mem.hugepagesz,
         		ss->mem.tothugepage,
-        		ss->mem.freehugepage);
+        		ss->mem.freehugepage,
+        		ss->mem.zfsarcsize);
 }
 
 void
@@ -448,7 +450,7 @@ print_SWP(char *hp, struct sstat *ss, struct tstat *ps, int nact)
 			pagesize,
 			ss->mem.totswap,
 			ss->mem.freeswap,
-			(long long)0,
+			ss->mem.swapcached,
 			ss->mem.committed,
 			ss->mem.commitlim);
 }
@@ -599,7 +601,7 @@ print_NET(char *hp, struct sstat *ss, struct tstat *ps, int nact)
 {
 	register int 	i;
 
-	printf(	"%s %s %lld %lld %lld %lld %lld %lld %lld %lld\n",
+	printf(	"%s %s %lld %lld %lld %lld %lld %lld %lld %lld %lld %lld %lld %lld %lld %lld %lld %lld\n",
 			hp,
 			"upper",
         		ss->net.tcp.InSegs,
@@ -615,7 +617,17 @@ print_NET(char *hp, struct sstat *ss, struct tstat *ps, int nact)
        		 	ss->net.ipv4.InDelivers +
        		 		ss->net.ipv6.Ip6InDelivers,
        		 	ss->net.ipv4.ForwDatagrams +
-       		 		ss->net.ipv6.Ip6OutForwDatagrams);
+       		 		ss->net.ipv6.Ip6OutForwDatagrams,
+       		 	ss->net.udpv4.InErrors +
+				ss->net.udpv6.Udp6InErrors,
+       		 	ss->net.udpv4.NoPorts +
+				ss->net.udpv6.Udp6NoPorts,
+			ss->net.tcp.ActiveOpens,
+			ss->net.tcp.PassiveOpens,
+			ss->net.tcp.CurrEstab,
+			ss->net.tcp.RetransSegs,
+			ss->net.tcp.InErrs,
+			ss->net.tcp.OutRsts);
 
 	for (i=0; ss->intf.intf[i].name[0]; i++)
 	{
@@ -668,7 +680,7 @@ print_PRG(char *hp, struct sstat *ss, struct tstat *ps, int nact)
 			exitcode = (ps->gen.excode >>   8) & 0xff;
 
 		printf("%s %d (%s) %c %d %d %d %d %d %ld (%s) %d %d %d %d "
- 		       "%d %d %d %d %d %d %ld %c %d %d %s\n",
+ 		       "%d %d %d %d %d %d %ld %c %d %d %s %c\n",
 			hp,
 			ps->gen.pid,
 			ps->gen.name,
@@ -694,7 +706,8 @@ print_PRG(char *hp, struct sstat *ss, struct tstat *ps, int nact)
 			ps->gen.isproc ? 'y':'n',
 			ps->gen.vpid,
 			ps->gen.ctid,
-			ps->gen.container[0] ? ps->gen.container:"-");
+			ps->gen.container[0] ? ps->gen.container:"-",
+        		ps->gen.excode & ~(INT_MAX) ? 'N' : '-');
 	}
 }
 
@@ -705,7 +718,8 @@ print_PRC(char *hp, struct sstat *ss, struct tstat *ps, int nact)
 
 	for (i=0; i < nact; i++, ps++)
 	{
-		printf("%s %d (%s) %c %u %lld %lld %d %d %d %d %d %d %d %c\n",
+		printf("%s %d (%s) %c %u %lld %lld %d %d %d %d %d %d %d %c "
+		       "%llu (%s)\n",
 				hp,
 				ps->gen.pid,
 				ps->gen.name,
@@ -720,7 +734,9 @@ print_PRC(char *hp, struct sstat *ss, struct tstat *ps, int nact)
 				ps->cpu.curcpu,
 				ps->cpu.sleepavg,
 				ps->gen.tgid,
-				ps->gen.isproc ? 'y':'n');
+				ps->gen.isproc ? 'y':'n',
+				ps->cpu.rundelay,
+				ps->cpu.wchan);
 	}
 }
 
@@ -732,7 +748,7 @@ print_PRM(char *hp, struct sstat *ss, struct tstat *ps, int nact)
 	for (i=0; i < nact; i++, ps++)
 	{
 		printf("%s %d (%s) %c %u %lld %lld %lld %lld %lld %lld "
-		       "%lld %lld %lld %lld %lld %d %c %lld\n",
+		       "%lld %lld %lld %lld %lld %d %c %lld %lld\n",
 				hp,
 				ps->gen.pid,
 				ps->gen.name,
@@ -752,7 +768,8 @@ print_PRM(char *hp, struct sstat *ss, struct tstat *ps, int nact)
 				ps->gen.tgid,
 				ps->gen.isproc ? 'y':'n',
 				ps->mem.pmem == (unsigned long long)-1LL ?
-								0:ps->mem.pmem);
+								0:ps->mem.pmem,
+				ps->mem.vlock);
 	}
 }
 
