@@ -136,8 +136,6 @@
 **
 */
 
-static const char rcsid[] = "$Id: photoproc.c,v 1.33 2010/04/23 12:19:35 gerlof Exp $";
-
 #include <sys/types.h>
 #include <sys/param.h>
 #include <dirent.h>
@@ -168,6 +166,8 @@ static int	procio(struct tstat *);
 static int	proccont(struct tstat *);
 static void	proccmd(struct tstat *);
 static void	procsmaps(struct tstat *);
+static void	procwchan(struct tstat *);
+static count_t	procschedstat(struct tstat *);
 
 unsigned long
 photoproc(struct tstat *tasklist, int maxtask)
@@ -269,6 +269,7 @@ photoproc(struct tstat *tasklist, int maxtask)
 			continue;
 		}
 
+		procschedstat(curtask);		/* from /proc/pid/schedstat */
 		proccmd(curtask);		/* from /proc/pid/cmdline */
 		dockstat += proccont(curtask);	/* from /proc/pid/cpuset  */
 
@@ -280,6 +281,13 @@ photoproc(struct tstat *tasklist, int maxtask)
 		if (calcpss)
 			procsmaps(curtask);	/* from /proc/pid/smaps */
 
+		/*
+		** determine thread's wchan, if wanted ('expensive' from
+		** a CPU consumption point-of-view)
+		*/
+                if (getwchan)
+                	procwchan(curtask);
+
 		// read network stats from netatop
 		netatop_gettask(curtask->gen.tgid, 'g', curtask);
 
@@ -297,6 +305,13 @@ photoproc(struct tstat *tasklist, int maxtask)
 			curtask->gen.nthrrun  = 0;
 			curtask->gen.nthrslpi = 0;
 			curtask->gen.nthrslpu = 0;
+
+			/*
+ 			** rundelay on process level is equal to the rundelay
+			** of the main thread; totalize the rundelays of all
+			** threads
+			*/
+			curtask->cpu.rundelay = 0;
 			
 			/*
 			** open underlying task directory
@@ -345,6 +360,18 @@ photoproc(struct tstat *tasklist, int maxtask)
 						continue;
 					}
 
+					/*
+					** determine thread's wchan, if wanted
+ 					** ('expensive' from a CPU consumption
+					** point-of-view)
+					*/
+                			if (getwchan)
+                        			procwchan(curthr);
+
+					/* totalize rundelays of all threads */
+					curtask->cpu.rundelay +=
+						procschedstat(curthr);
+
 					strcpy(curthr->gen.container,
 						curtask->gen.container);
 
@@ -655,6 +682,12 @@ procstatus(struct tstat *curtask)
 			continue;
 		}
 
+		if (memcmp(line, "VmLck:", 6)==0)
+		{
+			sscanf(line, "VmLck: %lld", &(curtask->mem.vlock));
+			continue;
+		}
+
 		if (memcmp(line, "SigQ:", 5)==0)
 			break;
 	}
@@ -764,6 +797,31 @@ proccmd(struct tstat *curtask)
 }
 
 
+/*
+** determine the wait channel of a sleeping thread
+** i.e. the name of the kernel function in which the thread
+** has been put in sleep state)
+*/
+static void
+procwchan(struct tstat *curtask)
+{
+        FILE            *fp;
+        register int    nr = 0;
+
+        if ( (fp = fopen("wchan", "r")) != NULL)
+        {
+
+                nr = fread(curtask->cpu.wchan, 1,
+			sizeof(curtask->cpu.wchan)-1, fp);
+                if (nr < 0)
+                        nr = 0;
+                fclose(fp);
+        }
+
+        curtask->cpu.wchan[nr] = 0;
+}
+
+
 /*
 ** store the Docker container ID, retrieved from the 'cpuset'
 ** that might look like this:
@@ -856,6 +914,7 @@ procsmaps(struct tstat *curtask)
 
 		procsmaps_firstcall = 0;
 	}
+
 	/*
  	** open the file (always succeeds, even if no root privs)
 	*/
@@ -891,3 +950,47 @@ procsmaps(struct tstat *curtask)
 	if (! droprootprivs())
 		mcleanstop(42, "failed to drop root privs\n");
 }
+
+/*
+** get run_delay from /proc/<pid>/schedstat
+** ref: https://git.kernel.org/pub/scm/linux/kernel/git/torvalds/linux.git/tree/Documentation/scheduler/sched-stats.rst?h=v5.7-rc6
+*/
+static count_t
+procschedstat(struct tstat *curtask)
+{
+	FILE	*fp;
+	char	line[4096];
+	count_t	runtime, rundelay = 0;
+	unsigned long pcount;
+	static char *schedstatfile = "schedstat";
+
+	/*
+ 	** open the schedstat file 
+	*/
+	if ( (fp = fopen(schedstatfile, "r")) )
+	{
+		curtask->cpu.rundelay = 0;
+
+		if (fgets(line, sizeof line, fp))
+		{
+			sscanf(line, "%llu %llu %lu\n",
+					&runtime, &rundelay, &pcount);
+
+			curtask->cpu.rundelay = rundelay;
+		}
+
+		/*
+		** verify if fgets returned NULL due to error i.s.o. EOF
+		*/
+		if (ferror(fp))
+			curtask->cpu.rundelay = 0;
+
+		fclose(fp);
+	}
+	else
+	{
+		curtask->cpu.rundelay = 0;
+	}
+
+	return curtask->cpu.rundelay;
+}
