@@ -149,13 +149,12 @@
 **
 */
 
-static const char rcsid[] = "$Id: photosyst.c,v 1.38 2010/11/19 07:40:40 gerlof Exp $";
-
 #include <sys/types.h>
 #include <stdio.h>
 #include <string.h>
 #include <unistd.h>
 #include <stdlib.h>
+#include <errno.h>
 #include <regex.h>
 #include <sys/stat.h>
 #include <sys/times.h>
@@ -655,6 +654,7 @@ photosyst(struct sstat *si)
 	si->mem.shmem 		= (count_t) 0;
 	si->mem.totswap  	= (count_t)-1;
 	si->mem.freeswap 	= (count_t)-1;
+	si->mem.swapcached 	= (count_t) 0;
 	si->mem.committed 	= (count_t) 0;
 
 	if ( (fp = fopen("meminfo", "r")) != NULL)
@@ -740,6 +740,11 @@ photosyst(struct sstat *si)
 							cnts[0]*1024/pagesize;
 					}
 				}
+			else	if (strcmp("SwapCached:", nam) == EQ)
+				{
+					si->mem.swapcached  =
+							cnts[0]*1024/pagesize;
+				}
 			else	if (strcmp("Slab:", nam) == EQ)
 				{
 					si->mem.slabmem = cnts[0]*1024/pagesize;
@@ -798,6 +803,33 @@ photosyst(struct sstat *si)
 		fclose(fp);
 	}
 
+	/*
+	** ZFSonlinux: gather size of ARC cache in memory
+	** searching for:
+	** 	size       4    519101312
+	*/ 
+	si->mem.zfsarcsize = (count_t) 0;
+
+	if ( (fp = fopen("spl/kstat/zfs/arcstats", "r")) != NULL)
+	{
+		while ( fgets(linebuf, sizeof(linebuf), fp) != NULL)
+		{
+			nr = sscanf(linebuf,
+				"%s %lld %lld", nam, &cnts[0], &cnts[1]);
+
+			if (nr < 3)
+				continue;
+
+			if ( strcmp("size", nam) == EQ)
+			{
+				si->mem.zfsarcsize = cnts[1] / pagesize;
+				break;
+			}
+		}
+
+		fclose(fp);
+	}
+
 	/*
 	** gather network-related statistics
  	** 	- interface stats from the file /proc/net/dev
@@ -1589,7 +1621,7 @@ lvmmapname(unsigned int major, unsigned int minor,
 		DIR		*dirp;
 		struct dirent	*dentry;
 		struct stat	statbuf;
-		char		path[64];
+		char		path[PATH_MAX];
 
 		if ( (dirp = opendir(MAPDIR)) )
 		{
@@ -1828,7 +1860,7 @@ get_infiniband(struct ifbstat *si)
 
 	if (firstcall)
 	{
-		char		path[128], *p;
+		char		path[PATH_MAX], *p;
 		struct stat	statbuf;
 		struct dirent	*contdent, *portdent;
 		DIR		*contp, *portp;
@@ -1944,7 +1976,7 @@ static void
 ibprep(struct ibcachent *ibc)
 {
 	FILE	*fp;
-	char	path[128], linebuf[64], speedunit;
+	char	path[PATH_MAX], linebuf[64], speedunit;
 
 	// determine port rate and number of lanes
 	snprintf(path, sizeof path, "%s/ports/%d/rate", ibc->ibha, ibc->port); 
@@ -1955,27 +1987,33 @@ ibprep(struct ibcachent *ibc)
 		{
 			(void) sscanf(linebuf, "%lld %c%*s (%hdX",
 				&(ibc->rate), &speedunit, &(ibc->lanes));
+
+			// calculate megabits/second
+			switch (speedunit)
+			{
+			   case 'M':
+			   case 'm':
+				break;
+			   case 'G':
+			   case 'g':
+				ibc->rate *= 1000;
+				break;
+			   case 'T':
+			   case 't':
+				ibc->rate *= 1000000;
+				break;
+			}
+
+		}
+		else
+		{
+			ibc->lanes = 0;
+			ibc->rate = 0;
 		}
 
 		fclose(fp);
 	}
 
-	// calculate megabits/second
-	switch (speedunit)
-	{
-	   case 'M':
-	   case 'm':
-		break;
-	   case 'G':
-	   case 'g':
-		ibc->rate *= 1000;
-		break;
-	   case 'T':
-	   case 't':
-		ibc->rate *= 1000000;
-		break;
-	}
-
 	// build all pathnames to obtain the counters
 	// of this port later on
 	snprintf(path, sizeof path, "%s/ports/%d/counters/port_rcv_data",
@@ -2110,6 +2148,7 @@ getperfevents(struct cpustat *cs)
 {
 	static int	firstcall = 1, cpualloced, *fdi, *fdc;
 	int		i;
+	int 		liResult;
 
 	if (!enable_perfevents())
 		return;
@@ -2196,11 +2235,29 @@ getperfevents(struct cpustat *cs)
         {
 		if (*(fdi+i) != -1)
 		{
-                	read(*(fdi+i), &(cs->cpu[i].instr), sizeof(count_t));
+                	liResult = read(*(fdi+i), &(cs->cpu[i].instr), sizeof(count_t));
                         cs->all.instr += cs->cpu[i].instr;
+			if(liResult < 0)
+			{
+				char lcMessage[64];
 
-                	read(*(fdc+i), &(cs->cpu[i].cycle), sizeof(count_t));
+				snprintf(lcMessage, sizeof(lcMessage) - 1,
+				          "%s:%d - Error %d reading instr counters\n",
+				           __FILE__, __LINE__, errno);
+				fprintf(stderr, "%s", lcMessage);
+			}
+
+                	liResult = read(*(fdc+i), &(cs->cpu[i].cycle), sizeof(count_t));
                         cs->all.cycle += cs->cpu[i].cycle;
+			if(liResult < 0)
+			{
+				char lcMessage[64];
+
+				snprintf(lcMessage, sizeof(lcMessage) - 1,
+				          "%s:%d - Error %d reading cycle counters\n",
+				           __FILE__, __LINE__, errno );
+				fprintf(stderr, "%s", lcMessage);
+			}
 		}
         }
 }
