@@ -73,7 +73,9 @@ struct	memstat {
 	count_t	hugepagesz;	// huge page size (bytes)
 
 	count_t	vmwballoon;	// vmware claimed balloon pages
-	count_t	cfuture[8];	// reserved for future use
+	count_t	zfsarcsize;	// zfsonlinux ARC size (pages)
+	count_t swapcached;	// swap cache (pages)
+	count_t	cfuture[6];	// reserved for future use
 };
 
 /************************************************************************/
