@@ -37,6 +37,7 @@
 #include <stdlib.h>
 #include <signal.h>
 #include <ctype.h>
+#include <string.h>
 #include <sys/utsname.h>
 #include <string.h>
 #include <regex.h>
@@ -50,71 +51,10 @@
 #include "showgeneric.h"
 #include "photoproc.h"
 #include "photosyst.h"
+#include "rawlog.h"
 
 #define	BASEPATH	"/var/log/atop"  
 
-/*
-** structure which describes the raw file contents
-**
-** layout raw file:    rawheader
-**
-**                     rawrecord                           \
-**                     compressed system-level statistics   | sample 1
-**                     compressed process-level statistics /
-**
-**                     rawrecord                           \
-**                     compressed system-level statistics   | sample 2
-**                     compressed process-level statistics /
-**
-** etcetera .....
-*/
-#define	MYMAGIC		(unsigned int) 0xfeedbeef
-
-struct rawheader {
-	unsigned int	magic;
-
-	unsigned short	aversion;	/* creator atop version with MSB */
-	unsigned short	future1;	/* can be reused 		 */
-	unsigned short	future2;	/* can be reused 		 */
-	unsigned short	rawheadlen;	/* length of struct rawheader    */
-	unsigned short	rawreclen;	/* length of struct rawrecord    */
-	unsigned short	hertz;		/* clock interrupts per second   */
-	unsigned short	sfuture[6];	/* future use                    */
-	unsigned int	sstatlen;	/* length of struct sstat        */
-	unsigned int	tstatlen;	/* length of struct tstat        */
-	struct utsname	utsname;	/* info about this system        */
-	char		cfuture[8];	/* future use                    */
-
-	unsigned int	pagesize;	/* size of memory page (bytes)   */
-	int		supportflags;  	/* used features                 */
-	int		osrel;		/* OS release number             */
-	int		osvers;		/* OS version number             */
-	int		ossub;		/* OS version subnumber          */
-	int		ifuture[6];	/* future use                    */
-};
-
-struct rawrecord {
-	time_t		curtime;	/* current time (epoch)         */
-
-	unsigned short	flags;		/* various flags                */
-	unsigned short	sfuture[3];	/* future use                   */
-
-	unsigned int	scomplen;	/* length of compressed sstat   */
-	unsigned int	pcomplen;	/* length of compressed tstat's */
-	unsigned int	interval;	/* interval (number of seconds) */
-	unsigned int	ndeviat;	/* number of tasks in list      */
-	unsigned int	nactproc;	/* number of processes in list  */
-	unsigned int	ntask;		/* total number of tasks        */
-	unsigned int    totproc;	/* total number of processes	*/
-	unsigned int    totrun;		/* number of running  threads	*/
-	unsigned int    totslpi;	/* number of sleeping threads(S)*/
-	unsigned int    totslpu;	/* number of sleeping threads(D)*/
-	unsigned int	totzomb;	/* number of zombie processes   */
-	unsigned int	nexit;		/* number of exited processes   */
-	unsigned int	noverflow;	/* number of overflow processes */
-	unsigned int	ifuture[6];	/* future use                   */
-};
-
 static int	getrawrec  (int, struct rawrecord *, int);
 static int	getrawsstat(int, struct sstat *, int);
 static int	getrawtstat(int, struct tstat *, int, int);
@@ -405,9 +345,9 @@ rawread(void)
 
 		if (lookslikedatetome(rawname))
 		{
-			char	savedname[RAWNAMESZ];
+			char	savedname[16];
 
-			strncpy(savedname, rawname, RAWNAMESZ-1);
+			strcpy(savedname, rawname); // no overflow (len=8)
 
 			snprintf(rawname, RAWNAMESZ, "%s/atop_%s",
 				BASEPATH, 
@@ -475,7 +415,7 @@ rawread(void)
 	*/
 	if ( (rawfd = open(rawname, O_RDONLY)) == -1)
 	{
-		char	command[512], tmpname1[RAWNAMESZ], tmpname2[RAWNAMESZ];
+		char	command[512], tmpname1[200], tmpname2[200];
 
 		/*
 		** check if a compressed raw file is present
@@ -514,6 +454,10 @@ rawread(void)
 		}
 	}
 
+	/* make the kernel readahead more effective, */
+	if (isregular)
+		posix_fadvise(rawfd, 0, 0, POSIX_FADV_SEQUENTIAL);
+
 	/*
 	** read the raw header and verify the magic
 	*/
@@ -610,9 +554,23 @@ rawread(void)
 	{
 		while ( getrawrec(rawfd, &rr, rh.rawreclen) == rh.rawreclen)
 		{
-			unsigned int	secsinday = daysecs(rr.curtime);
 			unsigned int	k, l;
 
+			cursortime = rr.curtime;	// maintain current
+
+			/*
+			** normalize the begintime and endtime if the
+			** format hh:mm has been used instead of an
+			** absolute date-time string
+			** (only happens for the first record)
+			*/
+			if (begintime <= SECONDSINDAY)
+				begintime = normalize_epoch(cursortime,
+								begintime);
+
+			if (endtime && endtime <= SECONDSINDAY)
+				endtime = normalize_epoch(cursortime, endtime);
+
 			/*
 			** store the offset of the raw record in the offset list
 			** in case of offset list overflow, extend the list
@@ -638,14 +596,37 @@ rawread(void)
 			** check if this sample is within the time-range
 			** specified with the -b and -e flags (if any)
 			*/
-			if ( (begintime && begintime > secsinday) )
+			if ( (begintime > cursortime) )
 			{
 				lastcmd = 1;
 						
 				if (isregular)
 				{
-					lseek(rawfd, rr.scomplen+rr.pcomplen,
-								SEEK_CUR);
+					static off_t curr_pos = -1;
+					off_t next_pos;
+
+					lastcmd = 1;
+					next_pos = lseek(rawfd, rr.scomplen+rr.pcomplen, SEEK_CUR);
+					if ((curr_pos >> READAHEADOFF) != (next_pos >> READAHEADOFF))
+					{
+						int liResult;
+						/* just read READAHEADSIZE bytes into page cache */
+						char *buf = malloc(READAHEADSIZE);
+						ptrverify(buf, "Malloc failed for readahead");
+						liResult = pread(rawfd, buf, READAHEADSIZE, next_pos & ~(READAHEADSIZE - 1));
+						if(liResult == -1)
+						{
+							char lcMessage[64];
+
+							snprintf(lcMessage, sizeof(lcMessage) - 1,
+								  "%s:%d - Error %d in readahead\n",
+							          __FILE__, __LINE__, errno);
+							fprintf(stderr, "%s", lcMessage);
+						}
+						free(buf);
+					}
+					curr_pos = next_pos;
+					continue;
 				}
 				else	// named pipe not seekable
 				{
@@ -666,7 +647,7 @@ rawread(void)
 
 			begintime = 0;	// allow earlier times from now on
 
-			if ( (endtime && endtime < secsinday) )
+			if ( (endtime && endtime < cursortime) )
 			{
 				if (isregular)
 					free(offlist);
@@ -696,7 +677,8 @@ rawread(void)
 				devtstat.procall = malloc(sizeof(struct tstat *)
 								* rr.totproc);
 
-			devtstat.procactive = malloc(sizeof(struct tstat *) * rr.nactproc);
+			devtstat.procactive = malloc(sizeof(struct tstat *) *
+								rr.nactproc);
 
 			ptrverify(devtstat.taskall,
 			          "Malloc failed for %d stored tasks\n",
@@ -800,9 +782,10 @@ rawread(void)
 			             rr.nexit, rr.noverflow, flags);
 			}
 			while (!isregular &&
-					(lastcmd == MSAMPPREV	||
-					 lastcmd == MSAMPBRANCH	||
-					 lastcmd == MRESET        ));
+				( lastcmd == MSAMPPREV		||
+				  lastcmd == MRESET     	||
+				 (lastcmd == MSAMPBRANCH &&
+						begintime < cursortime) ));
 
 			free(devtstat.taskall);
 			free(devtstat.procall);
@@ -825,10 +808,9 @@ rawread(void)
 				break;
 
 			   case MSAMPBRANCH:
-				if (begintime && begintime < secsinday)
+				if (begintime < cursortime && isregular)
 				{
-					lseek(rawfd, *offlist,
-							SEEK_SET);
+					lseek(rawfd, *offlist, SEEK_SET);
 					offcur = 1;
 				}
 			}
