@@ -255,8 +255,6 @@
 **
 */
 
-static const char rcsid[] = "$Id: showgeneric.c,v 1.71 2010/10/25 19:08:32 gerlof Exp $";
-
 #include <sys/types.h>
 #include <sys/param.h>
 #include <sys/stat.h>
@@ -274,6 +272,7 @@ static const char rcsid[] = "$Id: showgeneric.c,v 1.71 2010/10/25 19:08:32 gerlo
 #include <pwd.h>
 #include <grp.h>
 #include <regex.h>
+#include <locale.h>
 
 #include "atop.h"
 #include "photoproc.h"
@@ -289,6 +288,7 @@ static void	showhelp(int);
 static int	paused;     	/* boolean: currently in pause-mode     */
 static int	fixedhead;	/* boolean: fixate header-lines         */
 static int	sysnosort;	/* boolean: suppress sort of resources  */
+static int	threadsort;	/* boolean: sort threads per process    */
 static int	avgval;		/* boolean: average values i.s.o. total */
 static int	suppressexit;	/* boolean: suppress exited processes   */
 
@@ -351,7 +351,7 @@ generic_samp(time_t curtime, int nsecs,
 	register int	i, curline, statline, nproc;
 	int		firstproc = 0, plistsz, alistsz, killpid, killsig;
 	int		lastchar;
-	char		format1[16], format2[16], hhmm[16];
+	char		format1[16], format2[16], branchtime[32];
 	char		*statmsg = NULL, statbuf[80], genline[80];
 	char		 *lastsortp, curorder, autoorder;
 	char		buf[33];
@@ -507,21 +507,23 @@ generic_samp(time_t curtime, int nsecs,
 
                 int seclen	= val2elapstr(nsecs, buf);
                 int lenavail 	= (screen ? COLS : linelen) -
-						49 - seclen - utsnodenamelen;
+						51 - seclen - utsnodenamelen;
                 int len1	= lenavail / 3;
                 int len2	= lenavail - len1 - len1; 
 
-		printg("ATOP - %s%*s%s  %s%*s%c%c%c%c%c%c%c%c%c%c%c%c%c%c%*s%s"
-		       " elapsed", 
+		printg("ATOP - %s%*s%s  %s%*s%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%"
+		       "*s%s elapsed", 
 			utsname.nodename, len1, "", 
 			format1, format2, len1, "",
 			threadview                    ? MTHREAD    : '-',
+			threadsort                    ? MTHRSORT   : '-',
 			fixedhead  		      ? MSYSFIXED  : '-',
 			sysnosort  		      ? MSYSNOSORT : '-',
 			deviatonly 		      ? '-'        : MALLPROC,
 			usecolors  		      ? '-'        : MCOLORS,
 			avgval     		      ? MAVGVAL    : '-',
 			calcpss     		      ? MCALCPSS   : '-',
+			getwchan     		      ? MGETWCHAN  : '-',
 			suppressexit 		      ? MSUPEXITS  : '-',
 			procsel.userid[0] != USERSTUB ? MSELUSER   : '-',
 			procsel.prognamesz	      ? MSELPROC   : '-',
@@ -935,16 +937,22 @@ generic_samp(time_t curtime, int nsecs,
 				else
 					j = ntotal;
 
+				/*
+ 				** zip process list with thread list
+				*/
 				if (zipagain)
 				{
 					struct tstat *tall = devtstat->taskall;
 					struct tstat *pcur;
+					long int     n;
 
 					for (i=j=0; i < ncurlist; i++)
 					{
 					    pcur = curlist[i];
 
-					    tsklist[j++] = pcur;
+					    tsklist[j++] = pcur; // take process
+
+					    n = j; // start index of threads
 
 					    for (t = pcur - tall + 1;
 					         t < devtstat->ntaskall &&
@@ -965,6 +973,14 @@ generic_samp(time_t curtime, int nsecs,
 						else
 							tsklist[j++] = tall+t;
 					    }
+
+				            if (threadsort && j-n > 0 &&
+							curorder != MSORTMEM)
+					    {
+						qsort(&tsklist[n], j-n,
+				                  sizeof(struct tstat *),
+				                  procsort[(int)curorder&0x1f]);
+					    }
 					}
 
 					zipagain = 0;
@@ -1121,19 +1137,22 @@ generic_samp(time_t curtime, int nsecs,
                                 echo();
                                 move(statline, 0);
                                 clrtoeol();
-                                printw("Enter new time (format hh:mm): ");
+                                printw("Enter new time "
+				       "(format [YYYYMMDD]hhmm): ");
 
-                                hhmm[0] = '\0';
-                                scanw("%15s\n", hhmm);
+                                branchtime[0] = '\0';
+                                scanw("%31s\n", branchtime);
                                 noecho();
 
-                                if ( !hhmm2secs(hhmm, &begintime) )
+				begintime = cursortime;
+
+                                if ( !getbranchtime(branchtime, &begintime) )
                                 {
                                         move(statline, 0);
                                         clrtoeol();
                                         statmsg = "Wrong time format!";
                                         beep();
-                                        begintime = 0;
+					begintime = 0;
                                         break;
                                 }
 
@@ -1962,7 +1981,7 @@ generic_samp(time_t curtime, int nsecs,
 
 			   /*
 			   ** per-thread view wanted with sorting on
-			   ** process level or thread level
+			   ** process level
 			   */
 			   case MTHREAD:
 				if (threadview)
@@ -1979,6 +1998,24 @@ generic_samp(time_t curtime, int nsecs,
 				}
 				break;
 
+			   /*
+			   ** sorting on thread level as well (threadview)
+			   */
+			   case MTHRSORT:
+				if (threadsort)
+				{
+					threadsort = 0;
+					statmsg    = "Thread sorting disabled for thread view";
+					firstproc  = 0;
+				}
+				else
+				{
+					threadsort = 1;
+					statmsg    = "Thread sorting enabled for thread view";
+					firstproc  = 0;
+				}
+				break;
+
 			   /*
 			   ** per-process PSS calculation wanted 
 			   */
@@ -1995,6 +2032,22 @@ generic_samp(time_t curtime, int nsecs,
 				}
 				break;
 
+			   /*
+			   ** per-thread WCHAN definition 
+			   */
+			   case MGETWCHAN:
+				if (getwchan)
+				{
+					getwchan   = 0;
+					statmsg    = "WCHAN gathering disabled";
+				}
+				else
+				{
+					getwchan   = 1;
+					statmsg    = "WCHAN gathering enabled";
+				}
+				break;
+
 			   /*
 			   ** suppression of exited processes in output
 			   */
@@ -2413,6 +2466,7 @@ accumulate(struct tstat *curproc, struct tstat *curstat)
 		curstat->mem.vdata  += curproc->mem.vdata;
 		curstat->mem.vstack += curproc->mem.vstack;
 		curstat->mem.vswap  += curproc->mem.vswap;
+		curstat->mem.vlock  += curproc->mem.vlock;
 		curstat->mem.rgrow  += curproc->mem.rgrow;
 		curstat->mem.vgrow  += curproc->mem.vgrow;
 
@@ -2774,6 +2828,13 @@ generic_init(void)
 				threadview = 1;
 			break;
 
+		   case MTHRSORT:
+			if (threadsort)
+				threadsort = 0;
+			else
+				threadsort = 1;
+			break;
+
 		   case MCALCPSS:
 			if (calcpss)
 				calcpss = 0;
@@ -2781,6 +2842,13 @@ generic_init(void)
 				calcpss = 1;
 			break;
 
+		   case MGETWCHAN:
+			if (getwchan)
+				getwchan = 0;
+			else
+				getwchan = 1;
+			break;
+
 		   case MSUPEXITS:
 			if (suppressexit)
 				suppressexit = 0;
@@ -2824,9 +2892,23 @@ generic_init(void)
        	*/
        	if (screen)
 	{
+		/*
+		** if stdin is not connected to a tty (might be redirected
+		** to pipe or file), close it and duplicate stdout (tty)
+		** to stdin
+		*/
+       		if ( !isatty(0) )
+		{
+			(void) close(0);
+			(void) dup(1);
+		}
+
 		/*
 		** initialize screen-handling via curses
 		*/
+		setlocale(LC_ALL, "");
+		setlocale(LC_NUMERIC, "C");
+
 		initscr();
 		cbreak();
 		noecho();
@@ -2922,8 +3004,10 @@ static struct helptext {
 									' '},
 	{"\n",							' '},
 	{"Presentation (keys shown in header line):\n",  	' '},
-	{"\t'%c'  - show individual threads                        (toggle)\n",
+	{"\t'%c'  - show threads within process (thread view)      (toggle)\n",
 		 						MTHREAD},
+	{"\t'%c'  - sort threads (when combined with thread view)  (toggle)\n",
+		 						MTHRSORT},
 	{"\t'%c'  - show all processes (default: active processes) (toggle)\n",
 								MALLPROC},
 	{"\t'%c'  - show fixed number of header lines              (toggle)\n",
@@ -2938,6 +3022,8 @@ static struct helptext {
 								MAVGVAL},
 	{"\t'%c'  - calculate proportional set size (PSIZE)        (toggle)\n",
 								MCALCPSS},
+	{"\t'%c'  - determine WCHAN per thread                     (toggle)\n",
+								MGETWCHAN},
 	{"\n",							' '},
 	{"Raw file viewing:\n",					' '},
 	{"\t'%c'  - show next     sample in raw file\n",	MSAMPNEXT},
@@ -3057,7 +3143,8 @@ generic_usage(void)
 			MSUPEXITS);
 	printf("\t  -%c  show limited number of lines for certain resources\n",
 			MSYSLIMIT);
-	printf("\t  -%c  show individual threads\n", MTHREAD);
+	printf("\t  -%c  show threads within process\n", MTHREAD);
+	printf("\t  -%c  sort threads (when combined with '%c')\n", MTHRSORT, MTHREAD);
 	printf("\t  -%c  show average-per-second i.s.o. total values\n\n",
 			MAVGVAL);
 	printf("\t  -%c  no colors in case of high occupation\n",
@@ -3410,6 +3497,10 @@ do_flags(char *name, char *val)
 			threadview = 1;
 			break;
 
+		   case MTHRSORT:
+			threadsort = 1;
+			break;
+
 		   case MCOLORS:
 			usecolors = 0;
 			break;
@@ -3418,6 +3509,10 @@ do_flags(char *name, char *val)
 			calcpss = 1;
 			break;
 
+		   case MGETWCHAN:
+			getwchan = 1;
+			break;
+
 		   case MSUPEXITS:
 			suppressexit = 1;
 			break;
