@@ -99,7 +99,9 @@ struct sselection {
 #define	MSORTAUTO	'A'
 
 #define	MTHREAD		'y'
+#define	MTHRSORT	'Y'
 #define	MCALCPSS	'R'
+#define	MGETWCHAN	'W'
 #define	MSUPEXITS	'G'
 #define	MCOLORS		'x'
 #define	MSYSFIXED	'f'
