@@ -261,8 +261,6 @@
 **
 */
 
-static const char rcsid[] = "$Id: showlinux.c,v 1.70 2010/10/23 14:04:12 gerlof Exp $";
-
 #include <sys/types.h>
 #include <sys/param.h>
 #include <sys/stat.h>
@@ -388,6 +386,8 @@ sys_printdef *memsyspdefs[] = {
 	&syspdef_BLANKBOX,
 	&syspdef_VMWBAL,
 	&syspdef_BLANKBOX,
+	&syspdef_ZFSARC,
+	&syspdef_BLANKBOX,
 	&syspdef_HUPTOT,
 	&syspdef_HUPUSE,
         0
@@ -395,6 +395,9 @@ sys_printdef *memsyspdefs[] = {
 sys_printdef *swpsyspdefs[] = {
 	&syspdef_SWPTOT,
 	&syspdef_SWPFREE,
+	&syspdef_BLANKBOX,
+	&syspdef_SWPCACHE,
+	&syspdef_BLANKBOX,
 	&syspdef_SWPCOMMITTED,
 	&syspdef_SWPCOMMITLIM,
 	&syspdef_BLANKBOX,
@@ -410,6 +413,11 @@ sys_printdef *pagsyspdefs[] = {
         0
 };
 sys_printdef *psisyspdefs[] = {
+	&syspdef_PSICPUSTOT,
+	&syspdef_PSIMEMSTOT,
+	&syspdef_PSIMEMFTOT,
+	&syspdef_PSIIOSTOT,
+	&syspdef_PSIIOFTOT,
 	&syspdef_PSICPUS,
 	&syspdef_PSIMEMS,
 	&syspdef_PSIMEMF,
@@ -546,6 +554,8 @@ proc_printdef *allprocpdefs[]=
 	&procprt_PPID,
 	&procprt_SYSCPU,
 	&procprt_USRCPU,
+	&procprt_RUNDELAY,
+	&procprt_WCHAN,
 	&procprt_VGROW,
 	&procprt_RGROW,
 	&procprt_MINFLT,
@@ -558,6 +568,7 @@ proc_printdef *allprocpdefs[]=
 	&procprt_VDATA,
 	&procprt_VSTACK,
 	&procprt_SWAPSZ,
+	&procprt_LOCKSZ,
 	&procprt_CMD,
 	&procprt_RUID,
 	&procprt_EUID,
@@ -1018,21 +1029,23 @@ pricumproc(struct sstat *sstat, struct devtstat *devtstat,
                 if (memline[0].f == 0)
                 {
                     make_sys_prints(memline, MAXITEMS,
-	                "MEMTOT:6 "
-	                "MEMFREE:7 "
-	                "MEMCACHE:5 "
-	                "MEMDIRTY:3 "
-	                "MEMBUFFER:5 "
-	                "MEMSLAB:5 "
-	                "RECSLAB:2 "
+	                "MEMTOT:8 "
+	                "MEMFREE:9 "
+	                "MEMCACHE:7 "
+	                "MEMDIRTY:5 "
+	                "MEMBUFFER:7 "
+	                "MEMSLAB:7 "
+	                "RECSLAB:4 "
 	                "BLANKBOX:0 "
 	                "SHMEM:4 "
-	                "SHMRSS:3 "
-	                "SHMSWP:1 "
+	                "SHMRSS:4 "
+	                "SHMSWP:3 "
 	                "BLANKBOX:0 "
-	                "VMWBAL:4 "
+	                "VMWBAL:5 "
 	                "BLANKBOX:0 "
-	                "HUPTOT:4 "
+	                "ZFSARC:6 "
+	                "BLANKBOX:0 "
+	                "HUPTOT:6 "
 	                "HUPUSE:3 ", memsyspdefs, "builtin memline");
                 }
                 if (swpline[0].f == 0)
@@ -1042,8 +1055,7 @@ pricumproc(struct sstat *sstat, struct devtstat *devtstat,
 	                "SWPFREE:4 "
 	                "BLANKBOX:0 "
 	                "BLANKBOX:0 "
-	                "BLANKBOX:0 "
-	                "BLANKBOX:0 "
+	                "SWPCACHE:2 "
 	                "BLANKBOX:0 "
 	                "BLANKBOX:0 "
 	                "SWPCOMMITTED:5 "
@@ -1066,13 +1078,16 @@ pricumproc(struct sstat *sstat, struct devtstat *devtstat,
                 if (psiline[0].f == 0)
                 {
                     make_sys_prints(psiline, MAXITEMS,
-	                "PSICPUS:3 "
-	                "PSIMEMS:3 "
+	                "PSICPUSTOT:7 "
+	                "PSIMEMSTOT:7 "
+	                "PSIMEMFTOT:8 "
+	                "PSIIOSTOT:7 "
+	                "PSIIOFTOT:8 "
+	                "PSICPUS:6 "
+	                "PSIMEMS:5 "
 	                "PSIMEMF:3 "
-	                "PSIIOS:3 "
-	                "PSIIOF:3 "
-	                "BLANKBOX:0 "
-	                "BLANKBOX:0 "
+	                "PSIIOS:4 "
+	                "PSIIOF:2 "
 	                "BLANKBOX:0 ", psisyspdefs, "builtin psiline");
                 }
                 if (contline[0].f == 0)
@@ -1270,15 +1285,15 @@ priphead(int curlist, int totlist, char *showtype, char *showorder,
 
                 make_proc_prints(memprocs, MAXITEMS, 
                         "PID:10 TID:3 MINFLT:2 MAJFLT:2 VSTEXT:4 VSLIBS:4 "
-			"VDATA:4 VSTACK:4 VSIZE:6 RSIZE:7 PSIZE:5 "
+			"VDATA:4 VSTACK:4 LOCKSZ:3 VSIZE:6 RSIZE:7 PSIZE:5 "
                         "VGROW:7 RGROW:8 SWAPSZ:5 RUID:1 EUID:0 "
                         "SORTITEM:9 CMD:10", 
                         "built-in memprocs");
 
                 make_proc_prints(schedprocs, MAXITEMS, 
-                        "PID:10 TID:6 CID:5 VPID:4 CTID:4 TRUN:7 TSLPI:7 "
-			"TSLPU:7 POLI:8 NICE:9 PRI:9 RTPR:9 CPUNR:8 ST:8 "
-			"EXC:8 S:8 SORTITEM:10 CMD:10", 
+                        "PID:10 TID:6 CID:4 VPID:3 CTID:3 TRUN:7 TSLPI:7 "
+			"TSLPU:7 POLI:8 NICE:9 PRI:5 RTPR:9 CPUNR:8 ST:8 "
+			"EXC:8 S:8 RDELAY:8 WCHAN:5 SORTITEM:10 CMD:10",
                         "built-in schedprocs");
 
                 make_proc_prints(dskprocs, MAXITEMS, 
@@ -1314,19 +1329,19 @@ priphead(int curlist, int totlist, char *showtype, char *showorder,
 
                 make_proc_prints(totusers, MAXITEMS, 
                         "NPROCS:10 SYSCPU:9 USRCPU:9 VSIZE:6 "
-                        "RSIZE:8 PSIZE:8 SWAPSZ:5 RDDSK:7 CWRDSK:7 "
+                        "RSIZE:8 PSIZE:8 LOCKSZ:3 SWAPSZ:5 RDDSK:7 CWRDSK:7 "
 			"RNET:6 SNET:6 SORTITEM:10 RUID:10", 
                         "built-in totusers");
 
                 make_proc_prints(totprocs, MAXITEMS, 
                         "NPROCS:10 SYSCPU:9 USRCPU:9 VSIZE:6 "
-                        "RSIZE:8 PSIZE:8 SWAPSZ:5 RDDSK:7 CWRDSK:7 "
+                        "RSIZE:8 PSIZE:8 LOCKSZ:3 SWAPSZ:5 RDDSK:7 CWRDSK:7 "
 			"RNET:6 SNET:6 SORTITEM:10 CMD:10", 
                         "built-in totprocs");
 
                 make_proc_prints(totconts, MAXITEMS, 
                         "NPROCS:10 SYSCPU:9 USRCPU:9 VSIZE:6 "
-                        "RSIZE:8 PSIZE:8 SWAPSZ:5 RDDSK:7 CWRDSK:7 "
+                        "RSIZE:8 PSIZE:8 LOCKSZ:3 SWAPSZ:5 RDDSK:7 CWRDSK:7 "
 			"RNET:6 SNET:6 SORTITEM:10 CID:10", 
                         "built-in totconts");
         }
@@ -1412,10 +1427,11 @@ priphead(int curlist, int totlist, char *showtype, char *showorder,
 #define	FORMTID	"TID:6 "
 #define	FORMCID	"CID:5 "
 #define	FORMCPU	"SYSCPU:9 USRCPU:9 "
+#define	FORMDEL	"RDELAY:4 "
 #define FORMMEM	"VGROW:8 RGROW:8 "
 #define FORMDSK	"RDDSK:7 CWRDSK:7 "
 #define FORMNET	"RNET:6 SNET:6 "
-#define FORMMSC	"RUID:3 EUID:2 ST:4 EXC:4 THR:4 S:4 CPUNR:4 "
+#define FORMMSC	"RUID:2 EUID:1 ST:3 EXC:3 THR:3 S:3 CPUNR:3 "
 #define FORMEND	"SORTITEM:10 CMD:10"
 
 static void
@@ -1441,6 +1457,9 @@ make_proc_dynamicgen()
 	memcpy(p, FORMCPU, sizeof FORMCPU -1);
 	p += sizeof FORMCPU -1;
 
+	memcpy(p, FORMDEL, sizeof FORMDEL -1);
+	p += sizeof FORMDEL -1;
+
 	memcpy(p, FORMMEM, sizeof FORMMEM -1);
 	p += sizeof FORMMEM -1;
 
@@ -2280,9 +2299,13 @@ compcpu(const void *a, const void *b)
         register count_t bcpu = (*(struct tstat **)b)->cpu.stime +
                                 (*(struct tstat **)b)->cpu.utime;
 
-        if (acpu < bcpu) return  1;
-        if (acpu > bcpu) return -1;
-                         return compmem(a, b);
+        if (acpu < bcpu)
+                return  1;
+
+        if (acpu > bcpu)
+                return -1;
+
+        return compmem(a, b);
 }
 
 int
@@ -2304,9 +2327,13 @@ compdsk(const void *a, const void *b)
 	else
 		bdsk = tb->dsk.rio;
 
-        if (adsk < bdsk) return  1;
-        if (adsk > bdsk) return -1;
-                         return compcpu(a, b);
+        if (adsk < bdsk)
+                return  1;
+
+        if (adsk > bdsk)
+                return -1;
+
+        return compcpu(a, b);
 }
 
 int
@@ -2315,9 +2342,13 @@ compmem(const void *a, const void *b)
         register count_t amem = (*(struct tstat **)a)->mem.rmem;
         register count_t bmem = (*(struct tstat **)b)->mem.rmem;
 
-        if (amem < bmem) return  1;
-        if (amem > bmem) return -1;
-                         return  0;
+        if (amem < bmem)
+                return  1;
+
+        if (amem > bmem)
+                return -1;
+
+        return  0;
 }
 
 int
@@ -2338,15 +2369,23 @@ compgpu(const void *a, const void *b)
 
 	if (abusy == -1 || bbusy == -1)
 	{
-        	if (amem < bmem)	return  1;
-	        if (amem > bmem) 	return -1;
-                         		return  0;
+                if (amem < bmem)
+                        return  1;
+
+                if (amem > bmem)
+                        return -1;
+
+                return  0;
 	}
 	else
 	{
-		if (abusy < bbusy)	return  1;
-		if (abusy > bbusy)	return -1;
-       		                  	return  0;
+                if (abusy < bbusy)
+                        return  1;
+
+                if (abusy > bbusy)
+                        return -1;
+
+       		return  0;
 	}
 }
 
@@ -2362,9 +2401,13 @@ compnet(const void *a, const void *b)
                                 (*(struct tstat **)b)->net.udpssz +
                                 (*(struct tstat **)b)->net.udprsz  ;
 
-        if (anet < bnet) return  1;
-        if (anet > bnet) return -1;
-                         return compcpu(a, b);
+        if (anet < bnet)
+                return  1;
+
+        if (anet > bnet)
+                return -1;
+
+	return compcpu(a, b);
 }
 
 int
@@ -2373,9 +2416,13 @@ compusr(const void *a, const void *b)
         register int uida = (*(struct tstat **)a)->gen.ruid;
         register int uidb = (*(struct tstat **)b)->gen.ruid;
 
-        if (uida > uidb) return  1;
-        if (uida < uidb) return -1;
-                         return  0;
+        if (uida > uidb)
+                return  1;
+
+        if (uida < uidb)
+                return -1;
+
+        return  0;
 }
 
 int
@@ -2407,9 +2454,13 @@ cpucompar(const void *a, const void *b)
         register count_t bidle = ((struct percpu *)b)->itime +
                                  ((struct percpu *)b)->wtime;
 
-        if (aidle < bidle) return -1;
-        if (aidle > bidle) return  1;
-                           return  0;
+        if (aidle < bidle)
+                return -1;
+
+        if (aidle > bidle)
+                return  1;
+
+	return  0;
 }
 
 int
@@ -2422,15 +2473,23 @@ gpucompar(const void *a, const void *b)
 
 	if (agpuperc == -1 || bgpuperc == -1)
 	{
-        	if (amemuse < bmemuse)	return  1;
-        	if (amemuse > bmemuse)	return -1;
-                      			return  0;
+                if (amemuse < bmemuse)
+                        return  1;
+
+                if (amemuse > bmemuse)
+                        return -1;
+
+		return  0;
 	}
 	else
 	{
-        	if (agpuperc < bgpuperc)	return  1;
-        	if (agpuperc > bgpuperc)	return -1;
-                      				return  0;
+                if (agpuperc < bgpuperc)
+                        return  1;
+
+                if (agpuperc > bgpuperc)
+                        return -1;
+
+		return  0;
 	}
 }
 
@@ -2440,9 +2499,13 @@ diskcompar(const void *a, const void *b)
         register count_t amsio = ((struct perdsk *)a)->io_ms;
         register count_t bmsio = ((struct perdsk *)b)->io_ms;
 
-        if (amsio < bmsio) return  1;
-        if (amsio > bmsio) return -1;
-                           return  0;
+        if (amsio < bmsio)
+                return  1;
+
+        if (amsio > bmsio)
+                return -1;
+
+        return  0;
 }
 
 int
@@ -2488,16 +2551,24 @@ intfcompar(const void *a, const void *b)
         */
         if (aspeed && bspeed)
         {
-                if (afactor < bfactor)  return  1;
-                if (afactor > bfactor)  return -1;
-                                        return  0;
+                if (afactor < bfactor)
+                        return  1;
+
+                if (afactor > bfactor)
+                        return -1;
+
+                return  0;
         }
 
         if (!aspeed && !bspeed)
         {
-                if ((arbyte + asbyte) < (brbyte + bsbyte))      return  1;
-                if ((arbyte + asbyte) > (brbyte + bsbyte))      return -1;
-                                                                return  0;
+                if ((arbyte + asbyte) < (brbyte + bsbyte))
+                        return  1;
+
+                if ((arbyte + asbyte) > (brbyte + bsbyte))
+                        return -1;
+
+                return  0;
         }
 
         if (aspeed)
@@ -2515,9 +2586,13 @@ ifbcompar(const void *a, const void *b)
         count_t btransfer  = ((struct perifb *)b)->rcvb +
                              ((struct perifb *)b)->sndb;
 
-	if (atransfer < btransfer)	return  1;
-	if (atransfer > btransfer)	return -1;
-					return  0;
+	if (atransfer < btransfer)
+                return  1;
+
+	if (atransfer > btransfer)
+                return -1;
+
+	return  0;
 }
 
 
@@ -2536,9 +2611,13 @@ nfsmcompar(const void *a, const void *b)
 	                         nb->bytestotread + nb->bytestotwrite +
                                  nb->pagesmread   + nb->pagesmwrite;
 
-        if (aused < bused) return  1;
-        if (aused > bused) return -1;
-                           return  0;
+        if (aused < bused)
+                return  1;
+
+        if (aused > bused)
+                return -1;
+
+        return  0;
 }
 
 int
@@ -2550,9 +2629,13 @@ contcompar(const void *a, const void *b)
         register count_t aused = ca->system + ca->user + ca->nice;
         register count_t bused = cb->system + cb->user + cb->nice;
 
-        if (aused < bused) return  1;
-        if (aused > bused) return -1;
-                           return  0;
+        if (aused < bused) 
+                return  1;
+
+        if (aused > bused)
+                return -1;
+
+        return  0;
 }
 
 /*
