@@ -189,10 +189,12 @@ extern sys_printdef syspdef_SHMEM;
 extern sys_printdef syspdef_SHMRSS;
 extern sys_printdef syspdef_SHMSWP;
 extern sys_printdef syspdef_VMWBAL;
+extern sys_printdef syspdef_ZFSARC;
 extern sys_printdef syspdef_HUPTOT;
 extern sys_printdef syspdef_HUPUSE;
 extern sys_printdef syspdef_SWPTOT;
 extern sys_printdef syspdef_SWPFREE;
+extern sys_printdef syspdef_SWPCACHE;
 extern sys_printdef syspdef_SWPCOMMITTED;
 extern sys_printdef syspdef_SWPCOMMITLIM;
 extern sys_printdef syspdef_PAGSCAN;
@@ -200,6 +202,11 @@ extern sys_printdef syspdef_PAGSTEAL;
 extern sys_printdef syspdef_PAGSTALL;
 extern sys_printdef syspdef_PAGSWIN;
 extern sys_printdef syspdef_PAGSWOUT;
+extern sys_printdef syspdef_PSICPUSTOT;
+extern sys_printdef syspdef_PSIMEMSTOT;
+extern sys_printdef syspdef_PSIMEMFTOT;
+extern sys_printdef syspdef_PSIIOSTOT;
+extern sys_printdef syspdef_PSIIOFTOT;
 extern sys_printdef syspdef_PSICPUS;
 extern sys_printdef syspdef_PSIMEMS;
 extern sys_printdef syspdef_PSIMEMF;
@@ -314,6 +321,7 @@ extern proc_printdef procprt_VSLIBS;
 extern proc_printdef procprt_VDATA;
 extern proc_printdef procprt_VSTACK;
 extern proc_printdef procprt_SWAPSZ;
+extern proc_printdef procprt_LOCKSZ;
 extern proc_printdef procprt_CMD;
 extern proc_printdef procprt_RUID;
 extern proc_printdef procprt_EUID;
@@ -366,6 +374,8 @@ extern proc_printdef procprt_GPUMEMAVG;
 extern proc_printdef procprt_GPUGPUBUSY;
 extern proc_printdef procprt_GPUMEMBUSY;
 extern proc_printdef procprt_SORTITEM;
+extern proc_printdef procprt_RUNDELAY;
+extern proc_printdef procprt_WCHAN;
 
 
 
