@@ -82,8 +82,6 @@
 **
 */
 
-static const char rcsid[] = "$Id: showprocs.c,v 1.15 2011/09/05 11:44:16 gerlof Exp $";
-
 #include <sys/types.h>
 #include <sys/param.h>
 #include <sys/stat.h>
@@ -512,7 +510,10 @@ procprt_PPID_e(struct tstat *curstat, int avgval, int nsecs)
 {
         static char buf[64];
 
-	sprintf(buf, "%*s", procprt_PPID.width, "-");
+	if (curstat->gen.ppid)
+        	sprintf(buf, "%*d", procprt_PPID.width, curstat->gen.ppid);
+	else
+		sprintf(buf, "%*s", procprt_PPID.width, "-");
         return buf;
 }
 
@@ -819,6 +820,24 @@ proc_printdef procprt_SWAPSZ =
    { "SWAPSZ", "SWAPSZ", procprt_SWAPSZ_a, procprt_SWAPSZ_e, 6 };
 /***************************************************************/
 char *
+procprt_LOCKSZ_a(struct tstat *curstat, int avgval, int nsecs)
+{
+        static char buf[10];
+
+        val2memstr(curstat->mem.vlock*1024, buf, KBFORMAT, 0, 0);
+        return buf;
+}
+
+char *
+procprt_LOCKSZ_e(struct tstat *curstat, int avgval, int nsecs)
+{
+        return "    0K";
+}
+
+proc_printdef procprt_LOCKSZ = 
+   { "LOCKSZ", "LOCKSZ", procprt_LOCKSZ_a, procprt_LOCKSZ_e, 6 };
+/***************************************************************/
+char *
 procprt_CMD_a(struct tstat *curstat, int avgval, int nsecs)
 {
         static char buf[15];
@@ -2018,6 +2037,52 @@ proc_printdef procprt_GPUMEMBUSY =
    { "MEMBUSY", "GPUMEMBUSY", procprt_GPUMEMBUSY_ae, procprt_GPUMEMBUSY_ae, 7};
 /***************************************************************/
 char *
+procprt_WCHAN_a(struct tstat *curstat, int avgval, int nsecs)
+{
+        static char buf[32];
+
+        if (curstat->gen.state != 'R')
+		snprintf(buf, sizeof buf, "%-15.15s", curstat->cpu.wchan);
+	else
+		snprintf(buf, sizeof buf, "%-15.15s", " ");
+
+        return buf;
+}
+
+char *
+procprt_WCHAN_e(struct tstat *curstat, int avgval, int nsecs)
+{
+        static char buf[32];
+
+	snprintf(buf, sizeof buf, "%-15.15s", " ");
+        return buf;
+}
+
+proc_printdef procprt_WCHAN =
+   { "WCHAN          ", "WCHAN", procprt_WCHAN_a, procprt_WCHAN_e, 15 };
+/***************************************************************/
+char *
+procprt_RUNDELAY_a(struct tstat *curstat, int avgval, int nsecs)
+{
+        static char buf[10];
+
+        val2cpustr(curstat->cpu.rundelay/1000000, buf);
+        return buf;
+}
+
+char *
+procprt_RUNDELAY_e(struct tstat *curstat, int avgval, int nsecs)
+{
+        static char buf[10];
+
+        snprintf(buf, sizeof buf, "     -");
+        return buf;
+}
+
+proc_printdef procprt_RUNDELAY =
+   { "RDELAY", "RDELAY", procprt_RUNDELAY_a, procprt_RUNDELAY_e, 6 };
+/***************************************************************/
+char *
 procprt_SORTITEM_ae(struct tstat *curstat, int avgval, int nsecs)
 {
         return "";   // dummy function
