@@ -68,8 +68,6 @@
 **
 */
 
-static const char rcsid[] = "XXXXXX";
-
 #include <sys/types.h>
 #include <sys/param.h>
 #include <sys/stat.h>
@@ -961,9 +959,13 @@ char *
 sysprt_CPLAVG1(void *p, void *notused, int badness, int *color) 
 {
         struct sstat *sstat=p;
-        static char buf[15]="avg1 ";
+        static char buf[17]="avg1 ";
 
-        if (sstat->cpu.lavg1  > 999.0)
+        if (sstat->cpu.lavg1 > 999999.0)
+        {
+                sprintf(buf+5, ">999999");
+        }
+        else if (sstat->cpu.lavg1 > 999.0)
         {
                 sprintf(buf+5, "%7.0f", sstat->cpu.lavg1);
         }
@@ -982,7 +984,11 @@ sysprt_CPLAVG5(void *p, void *notused, int badness, int *color)
         struct sstat *sstat=p;
         static char buf[15]="avg5 ";
 
-        if (sstat->cpu.lavg5  > 999.0)
+        if (sstat->cpu.lavg5 > 999999.0)
+        {
+                sprintf(buf+5, ">999999");
+        }
+        else if (sstat->cpu.lavg5 > 999.0)
         {
                 sprintf(buf+5, "%7.0f", sstat->cpu.lavg5);
         }
@@ -1004,7 +1010,11 @@ sysprt_CPLAVG15(void *p, void *notused, int badness, int *color)
 	if (sstat->cpu.lavg15 > (2 * sstat->cpu.nrcpu) )
 		*color = COLORALMOST;
 
-        if (sstat->cpu.lavg15  > 999.0)
+        if (sstat->cpu.lavg15 > 99999.0)
+        {
+                sprintf(buf+6, ">99999");
+        }
+        else if (sstat->cpu.lavg15 > 999.0)
         {
                 sprintf(buf+6, "%6.0f", sstat->cpu.lavg15);
         }
@@ -1407,6 +1417,18 @@ sysprt_VMWBAL(void *p, void *notused, int badness, int *color)
 sys_printdef syspdef_VMWBAL = {"VMWBAL", sysprt_VMWBAL};
 /*******************************************************************/
 char *
+sysprt_ZFSARC(void *p, void *notused, int badness, int *color) 
+{
+        struct sstat *sstat=p;
+        static char buf[16]="zfarc  ";
+	*color = -1;
+        val2memstr(sstat->mem.zfsarcsize * pagesize, buf+6, MBFORMAT, 0, 0);
+        return buf;
+}
+
+sys_printdef syspdef_ZFSARC = {"ZFSARC", sysprt_ZFSARC};
+/*******************************************************************/
+char *
 sysprt_SWPTOT(void *p, void *notused, int badness, int *color) 
 {
         struct sstat *sstat=p;
@@ -1431,6 +1453,18 @@ sysprt_SWPFREE(void *p, void *notused, int badness, int *color)
 sys_printdef syspdef_SWPFREE = {"SWPFREE", sysprt_SWPFREE};
 /*******************************************************************/
 char *
+sysprt_SWPCACHE(void *p, void *notused, int badness, int *color) 
+{
+        struct sstat *sstat=p;
+        static char buf[16]="swcac ";
+	*color = -1;
+        val2memstr(sstat->mem.swapcached * pagesize, buf+6, MBFORMAT, 0, 0);
+        return buf;
+}
+
+sys_printdef syspdef_SWPCACHE = {"SWPCACHE", sysprt_SWPCACHE};
+/*******************************************************************/
+char *
 sysprt_SWPCOMMITTED(void *p, void *notused, int badness, int *color) 
 {
         struct sstat *sstat=p;
@@ -1521,9 +1555,9 @@ sysprt_PAGSWOUT(void *p, void *q, int badness, int *color)
 
 sys_printdef syspdef_PAGSWOUT = {"PAGSWOUT", sysprt_PAGSWOUT};
 /*******************************************************************/
-// general formatting of PSI field
+// general formatting of PSI field in avg10/avg60/avg300
 void
-psiformat(struct psi *p, char *head, char *buf, int bufsize)
+psiformatavg(struct psi *p, char *head, char *buf, int bufsize)
 {
 	static char	formats[] = "%.0f/%.0f/%.0f";
 	char		tmpbuf[32];
@@ -1554,7 +1588,7 @@ sysprt_PSICPUS(void *p, void *q, int badness, int *color)
 {
         struct sstat *sstat=p;
         static char buf[16];
-	psiformat(&(sstat->psi.cpusome), "cs", buf, sizeof buf);
+	psiformatavg(&(sstat->psi.cpusome), "cs", buf, sizeof buf);
         return buf;
 }
 sys_printdef syspdef_PSICPUS = {"PSICPUS", sysprt_PSICPUS};
@@ -1564,7 +1598,7 @@ sysprt_PSIMEMS(void *p, void *q, int badness, int *color)
 {
         struct sstat *sstat=p;
         static char buf[16];
-	psiformat(&(sstat->psi.memsome), "ms", buf, sizeof buf);
+	psiformatavg(&(sstat->psi.memsome), "ms", buf, sizeof buf);
         return buf;
 }
 sys_printdef syspdef_PSIMEMS = {"PSIMEMS", sysprt_PSIMEMS};
@@ -1574,7 +1608,7 @@ sysprt_PSIMEMF(void *p, void *q, int badness, int *color)
 {
         struct sstat *sstat=p;
         static char buf[16];
-	psiformat(&(sstat->psi.memfull), "mf", buf, sizeof buf);
+	psiformatavg(&(sstat->psi.memfull), "mf", buf, sizeof buf);
         return buf;
 }
 sys_printdef syspdef_PSIMEMF = {"PSIMEMF", sysprt_PSIMEMF};
@@ -1584,7 +1618,7 @@ sysprt_PSIIOS(void *p, void *q, int badness, int *color)
 {
         struct sstat *sstat=p;
         static char buf[16];
-	psiformat(&(sstat->psi.iosome), "is", buf, sizeof buf);
+	psiformatavg(&(sstat->psi.iosome), "is", buf, sizeof buf);
         return buf;
 }
 sys_printdef syspdef_PSIIOS = {"PSIIOS", sysprt_PSIIOS};
@@ -1594,11 +1628,85 @@ sysprt_PSIIOF(void *p, void *q, int badness, int *color)
 {
         struct sstat *sstat=p;
         static char buf[16];
-	psiformat(&(sstat->psi.iofull), "if", buf, sizeof buf);
+	psiformatavg(&(sstat->psi.iofull), "if", buf, sizeof buf);
         return buf;
 }
 sys_printdef syspdef_PSIIOF = {"PSIIOF", sysprt_PSIIOF};
 
+/*******************************************************************/
+// general formatting of PSI field in total percentage
+void
+psiformattot(struct psi *p, char *head, void *q, int *color, char *buf, int bufsize)
+{
+	static char	formats[] = "%-7.7s %3lu%%";
+        extraparam      *as=q;
+	unsigned long 	perc = p->total/(as->nsecs*10000);
+
+	if (perc > 100)
+		perc = 100;
+
+	if (perc >= 1)
+		*color = COLORALMOST;
+
+	snprintf(buf, bufsize, formats, head, perc);
+}
+
+char *
+sysprt_PSICPUSTOT(void *p, void *q, int badness, int *color) 
+{
+        struct sstat *sstat=p;
+        static char buf[32];
+
+	psiformattot(&(sstat->psi.cpusome), "cpusome", q, color, buf, sizeof buf);
+        return buf;
+}
+sys_printdef syspdef_PSICPUSTOT = {"PSICPUSTOT", sysprt_PSICPUSTOT};
+
+char *
+sysprt_PSIMEMSTOT(void *p, void *q, int badness, int *color) 
+{
+        struct sstat *sstat=p;
+        static char buf[32];
+
+	psiformattot(&(sstat->psi.memsome), "memsome", q, color, buf, sizeof buf);
+        return buf;
+}
+sys_printdef syspdef_PSIMEMSTOT = {"PSIMEMSTOT", sysprt_PSIMEMSTOT};
+
+char *
+sysprt_PSIMEMFTOT(void *p, void *q, int badness, int *color) 
+{
+        struct sstat *sstat=p;
+        static char buf[32];
+
+	psiformattot(&(sstat->psi.memfull), "memfull", q, color, buf, sizeof buf);
+        return buf;
+}
+sys_printdef syspdef_PSIMEMFTOT = {"PSIMEMFTOT", sysprt_PSIMEMFTOT};
+
+char *
+sysprt_PSIIOSTOT(void *p, void *q, int badness, int *color) 
+{
+        struct sstat *sstat=p;
+        static char buf[32];
+
+	psiformattot(&(sstat->psi.iosome), "iosome", q, color, buf, sizeof buf);
+        return buf;
+}
+sys_printdef syspdef_PSIIOSTOT = {"PSIIOSTOT", sysprt_PSIIOSTOT};
+
+
+char *
+sysprt_PSIIOFTOT(void *p, void *q, int badness, int *color) 
+{
+        struct sstat *sstat=p;
+        static char buf[32];
+
+	psiformattot(&(sstat->psi.iofull), "iofull", q, color, buf, sizeof buf);
+        return buf;
+}
+sys_printdef syspdef_PSIIOFTOT = {"PSIIOFTOT", sysprt_PSIIOFTOT};
+
 /*******************************************************************/
 char *
 sysprt_CONTNAME(void *p, void *q, int badness, int *color) 
@@ -1829,18 +1937,30 @@ sysprt_DSKAVIO(void *p, void *q, int badness, int *color)
 
 	*color = -1;
 
-        if (tim > 100.0)
+        if (tim >= 100.0)
         {
                 sprintf(buf+5, "%4.0lf ms", tim);
-        } 
-        else if (tim > 10.0) 
+        }
+        else if (tim >= 10.0)
         {
                 sprintf(buf+5, "%4.1lf ms", tim);
         }
-        else 
+        else if (tim >= 0.1)
         {
                 sprintf(buf+5, "%4.2lf ms", tim);
         }
+        else if (tim >= 0.01)
+        {
+                sprintf(buf+5, "%4.1lf µs", tim * 1000.0);
+        }
+        else if (tim >= 0.0001)
+        {
+                sprintf(buf+5, "%4.2lf µs", tim * 1000.0);
+        }
+        else
+        {
+                sprintf(buf+5, "%4.1lf ns", tim * 1000000.0);
+        }
 
         return buf;
 }
@@ -2124,6 +2244,15 @@ sysprt_NETNAME(void *p, void *q, int badness, int *color)
                                		(sstat->intf.intf[as->index].speed *10);
 		}
 
+		if( busy < -99 )
+		{
+			// when we get wrong values, show wrong values
+			busy = -99;
+		}		
+		else if( busy > 999 )
+		{
+			busy = 999;
+		}
 	        snprintf(buf, sizeof(buf)-1, "%-7.7s %3lld%%", 
        		          sstat->intf.intf[as->index].name, busy);
 
@@ -2204,7 +2333,17 @@ char *makenetspeed(count_t val, int nsecs)
                 c = 'T';
         }
 
-        sprintf(buf+3, "%4lld %cbps", val, c);
+        if(val < -999)
+        {
+		// when we get wrong values, show wrong values
+                val = -999;
+        }
+        else if(val > 9999)
+        {
+                val = 9999;
+        }
+
+        snprintf(buf+3, sizeof(buf)-3, "%4lld %cbps", val, c);
 
         return buf;
 }
@@ -2219,6 +2358,9 @@ sysprt_NETSPEEDMAX(void *p, void *q, int badness, int *color)
 
 	*color = -1;
 
+	if (speed < 0 )
+		speed = 0;
+
 	if (speed < 10000)
 	{
         	snprintf(buf, sizeof buf, "sp %4lld Mbps", speed);
@@ -2226,6 +2368,11 @@ sysprt_NETSPEEDMAX(void *p, void *q, int badness, int *color)
 	else
 	{
 		speed /= 1000;
+
+		if (speed > 9999)
+		{
+			speed = 9999;
+		}
         	snprintf(buf, sizeof buf, "sp %4lld Gbps", speed);
 	}
 
@@ -2412,7 +2559,7 @@ sysprt_IFBSPEEDMAX(void *p, void *q, int badness, int *color)
 {
         struct sstat *sstat = p;
         extraparam *as = q;
-        static char buf[16];
+        static char buf[64];
         count_t rate = sstat->ifb.ifb[as->index].rate;
 
 	*color = -1;
@@ -2424,6 +2571,11 @@ sysprt_IFBSPEEDMAX(void *p, void *q, int badness, int *color)
 	else
 	{
 		rate /= 1000;
+
+		if (rate > 9999)
+		{
+			rate = 9999;
+		}
         	snprintf(buf, sizeof buf, "sp %4lld Gbps", rate);
 	}
 
